/*
 
 * GrayStar
 * V2.0, Sept 2014
 * JQuery version
 * 
 * C. Ian Short
 * Saint Mary's University
 * Department of Astronomy and Physics
 * Institute for Computational Astrophysics (ICA)
 * Halifax, NS, Canada
 *  * ian.short@smu.ca
 * www.ap.smu.ca/~ishort/
 *
 * 1D, static, plane-parallel, LTE, gray stellar atmospheric model
 * core + wing approximation to Voigt spectral line profile
 *
 * Suitable for pedagogical purposes only
 * 
 * Logic written in Java SE 8.0, JDK 1.8
 * GUI written with JavaFX 8.0
 * 
 * Ported to JavaScript for deployment
 *
 * System requirements for Java version: Java run-time environment (JRE)
 * System requirements for JavaScript version: JavaScript intrepretation enabld in WWW browser (usually by default)
 *
 * Code provided "as is" - there is no formal support 
 *
 * Java default license applies:
 * End users may adapt, modify, develop, debug, and deploy at will
 * for academic and othe non-profit uses, but are asked to leave this
 * header text in place (although they may add to the header text).
 *
 */


// **********************************************

// Global variables - Doesn't work - scope not global!




var c = 2.9979249E+10; // light speed in vaccuum in cm/s
var sigma = 5.670373E-5; //Stefan-Boltzmann constant ergs/s/cm^2/K^4  
var k = 1.3806488E-16; // Boltzmann constant in ergs/K
var h = 6.62606957E-27; //Planck's constant in ergs sec
var ee = 4.80320425E-10; //fundamental charge unit in statcoulombs (cgs)
var mE = 9.10938291E-28; //electron mass (g)
//Conversion factors
var amu = 1.66053892E-24; // atomic mass unit in g
var eV = 1.602176565E-12; // eV in ergs

//Methods:
//Natural logs more useful than base 10 logs - Eg. Formal soln module: 
// Fundamental constants
var logC = Math.log(c);
var logSigma = Math.log(sigma);
var logK = Math.log(k);
var logH = Math.log(h);
var logEe = Math.log(ee); //Named so won't clash with log_10(e)
var logMe = Math.log(mE);
//Conversion factors
var logAmu = Math.log(amu);
var logEv = Math.log(eV);
// ********************************************

//***************************  Main ******************************


//***************************  Main ******************************



function main() {




//**********************************************************



// Intrinsic utility function:
//
// numPrint function to Set up special area of screen for printing out computed values for trouble-shooting 
// requires value to be printed, and the x and y pixel positions in that order
// Must be defined in scope of main() - ??

    var numPrint = function(val, x, y, r255, g255, b255, areaId) {

        var xStr = numToPxStrng(x);
        var yStr = numToPxStrng(y);
        var RGBHex = colHex(r255, g255, b255);
        var valStr = val.toString(10);
        var numId = document.createElement("p");
        numId.style.position = "absolute";
        numId.style.display = "block";
        numId.style.marginTop = yStr;
        numId.style.marginLeft = xStr;
        numId.style.color = RGBHex;
        numId.innerHTML = valStr;
        //masterId.appendChild(numId);
        areaId.appendChild(numId);
    }; // end numPrint

    var txtPrint = function(text, x, y, r255, g255, b255, areaId) {

        var xStr = numToPxStrng(x);
        var yStr = numToPxStrng(y);
        var RGBHex = colHex(r255, g255, b255);
        var txtId = document.createElement("p");
        txtId.style.position = "absolute";
        txtId.style.display = "block";
        txtId.style.width = "500px";
        txtId.style.marginTop = yStr;
        txtId.style.marginLeft = xStr;
        txtId.style.color = RGBHex;
        txtId.innerHTML = text;
        //masterId.appendChild(numId);
        areaId.appendChild(txtId);
    }; // end txtPrint


    /* 
     plotPnt takes in the *numerical* x- and y-coordinates, hexadecimal colour, and opacity, and plots a generic plotting dot at that location:
     Calls numToPxStrng to convert numeric coordinates and opacity to style attribute strings for HTML
     Calls colHex to convert R, G, and B amounts out of 255 into #RRGGBB hex format  
     */



    var plotPnt = function(x, y, r255, g255, b255, opac, dSize, areaId) {

        var xStr = numToPxStrng(x);
        var yStr = numToPxStrng(y);
        var opacStr = numToPxStrng(opac);
        var dSizeStr = numToPxStrng(dSize);
        var RGBHex = colHex(r255, g255, b255);
//   var RGBHex = "#000000";


// Each dot making up the line is a separate element:
        var dotId = document.createElement("div");
        dotId.class = "dot";
        dotId.style.position = "absolute";
        dotId.style.display = "block";
        dotId.style.height = dSizeStr;
        dotId.style.width = dSizeStr;
        dotId.style.borderRadius = "100%";
        dotId.style.opacity = opacStr;
        dotId.style.backgroundColor = RGBHex;
        dotId.style.marginLeft = xStr;
        dotId.style.marginTop = yStr;
//Append the dot to the plot
        //masterId.appendChild(dotId);
        areaId.appendChild(dotId);
    };
    /*
     colHex takes in red, green, and blue (in that order!) amounts out of 255 and converts
     them into stringified #RRGGBB format for HTML 
     */


    var colHex = function(r255, g255, b255) {



        var rr = Math.floor(r255); //MUST convert to integer
        var gg = Math.floor(g255); //MUST convert to integer
        var bb = Math.floor(b255); //MUST convert to integer

        var RGBHex = "rgb(";
        RGBHex = RGBHex.concat(rr);
        RGBHex = RGBHex.concat(",");
        RGBHex = RGBHex.concat(gg);
        RGBHex = RGBHex.concat(",");
        RGBHex = RGBHex.concat(bb);
        RGBHex = RGBHex.concat(")");
//////    var RGBHex = "rgb(60,80,120)";



        return RGBHex;
    };
// ***********************************

// Input control:



    // Get the checkbox values controlling what's plotted:
//JQuery test code:
//    $("#btnId").click(function() {
    //       alert("Value: " + $("#Teff").val());
    //       //alert("Value: ");
    //   });

    var settingsId = document.getElementById("settingsId");
    var btnId = document.getElementById("btnId");
    btnId.onClick = function() {
        settingsId = document.getElementById("settingsId"); // Still needed w. JQuery - settingId[] is useful
    };


// !!!! CAUTION !!!! This is a REALLY dangerous (but convenient) way to do it - the order of the switches in the html file CANNOT change!

    /*  If no JQuery...
     var teff = 1.0 * settingsId[0].value; // K
     var logg = 1.0 * settingsId[1].value; // log cgs
     var kappaScale = 1.0 * settingsId[2].value; // linear factor
     var massStar = 1.0 * settingsId[3].value; // solar masses
     
     var lam0 = 1.0 * settingsId[4].value; //nm
     //var logN = 1.0 * settingsId[5].value; // log total no. density absorbers, cgs  
     var A12 = 1.0 * settingsId[5].value; // A_12 logarithmic abundance = log_10(N/H_H) = 12
     var logF = 1.0 * settingsId[6].value; // log(f) oscillaotr strength // saturated line
     var chiI = 1.0 * settingsId[7].value; // ground state chi_I, eV
     var chiL = 1.0 * settingsId[8].value; // lower atomic E-level, eV     
     
     var xiT = 1.0 * settingsId[9].value; // km/s
     var mass = 1.0 * settingsId[10].value; //amu, Carbon
     var logGammaCol = 1.0 * settingsId[11].value; // log VanderWaals enhancement
     */

//JQuery:
    var teff = 1.0 * $("#Teff").val(); // K
    var logg = 1.0 * $("#logg").val(); // log cgs
    var kappaScale = 1.0 * $("#kappaScale").val(); // linear factor
    var massStar = 1.0 * $("#starMass").val(); // solar masses

    var lam0 = 1.0 * $("#lambda_0").val(); //nm
    var A12 = 1.0 * $("#A12").val(); // A_12 logarithmic abundance = log_10(N/H_H) = 12
    var logF = 1.0 * $("#logf").val(); // log(f) oscillaotr strength // saturated line
    var chiI = 1.0 * $("#chi_I").val(); // ground state chi_I, eV
    var chiL = 1.0 * $("#chi_L").val(); // lower atomic E-level, eV

    var xiT = 1.0 * $("#xi_T").val(); // km/s
    var mass = 1.0 * $("#mass").val(); //amu, Carbon
    var logGammaCol = 1.0 * $("#gammaCol").val(); // log VanderWaals enhancement

    var switchStar = "None";
    var numPreStars = 4;

    /*
     // None: (default)
     if (settingsId[12].checked) {
     switchStar = settingsId[12].value; // radio 
     }
     
     // Sun
     if (settingsId[13].checked) {
     switchStar = settingsId[13].value; // radio 
     }
     // Vega
     if (settingsId[14].checked) {
     switchStar = settingsId[14].value; // radio 
     }
     // Arcturus
     if (settingsId[15].checked) {
     switchStar = settingsId[15].value; // radio 
     }
     
     // Procyon
     if (settingsId[16].checked) {
     switchStar = settingsId[16].value; // radio 
     }
     */

    //JQuery:
    // None: (default)
    if ($("#none").is(":checked")) {
        switchStar = $("#none").val(); // radio 
    }

    // Sun
    if ($("#sun").is(":checked")) {
        switchStar = $("#sun").val(); // radio 
    }
    // Vega
    if ($("#vega").is(":checked")) {
        switchStar = $("#vega").val(); // radio 
    }
    // Arcturus
    if ($("#arcturus").is(":checked")) {
        switchStar = $("#arcturus").val(); // radio 
    }

    // Procyon
    if ($("#procyon").is(":checked")) {
        switchStar = $("#procyon").val(); // radio 
    }

    // $("#btnId").click(function() {
    //alert("switchStar: " + switchStar);
    //alert("switchStar: " + $("#Teff").val());
    //alert("Value: ");
    // });

    /*
     if (switchStar == "Sun") {
     var teff = 5778.0;
     settingsId[0].value = 5778.0;
     var logg = 4.44;
     settingsId[1].value = 4.44;
     var kappaScale = 1.0;
     settingsId[2].value = 1.0;
     var massStar = 1.0;
     settingsId[3].value = 1.0;
     }
     
     if (switchStar == "Arcturus") {
     var teff = 4250.0;
     settingsId[0].value = 4250.0;
     var logg = 2.0;
     settingsId[1].value = 2.0;
     var kappaScale = 0.333;
     settingsId[2].value = 0.333;
     var massStar = 1.1;
     settingsId[3].value = 1.1;
     }
     
     if (switchStar == "Vega") {
     var teff = 9550.0;
     settingsId[0].value = 9550.0;
     var logg = 3.95;
     settingsId[1].value = 3.95;
     var kappaScale = 0.333;
     settingsId[2].value = 0.333;
     var massStar = 2.1;
     settingsId[3].value = 2.1;
     }
     
     if (switchStar == "Procyon") {
     var teff = 6530.0;
     settingsId[0].value = 6530.0;
     var logg = 4.0;
     settingsId[1].value = 4.0;
     var kappaScale = 1.0;
     settingsId[2].value = 1.0;
     var massStar = 1.4;
     settingsId[3].value = 1.4;
     }
     */

    //JQuery:
    if (switchStar == "Sun") {
        var teff = 5778.0;
        settingsId[0].value = 5778.0;
        $("#Teff").val(5778.0);
        var logg = 4.44;
        settingsId[1].value = 4.44;
        $("#logg").val(4.44);
        var kappaScale = 1.0;
        settingsId[2].value = 1.0;
        $("#kappaScale").val(1.0);
        var massStar = 1.0;
        settingsId[3].value = 1.0;
        $("#starMass").val(1.0);
    }

    if (switchStar == "Arcturus") {
        var teff = 4250.0;
        settingsId[0].value = 4250.0;
        $("#Teff").val(4250.0);
        var logg = 2.0;
        settingsId[1].value = 2.0;
        $("#logg").val(2.0);
        var kappaScale = 0.333;
        settingsId[2].value = 0.333;
        $("#kappaScale").val(0.333);
        var massStar = 1.1;
        settingsId[3].value = 1.1;
        $("#starMass").val(1.1);
    }

    if (switchStar == "Vega") {
        var teff = 9550.0;
        settingsId[0].value = 9550.0;
        $("#Teff").val(9550.0);
        var logg = 3.95;
        settingsId[1].value = 3.95;
        $("#logg").val(3.95);
        var kappaScale = 0.333;
        settingsId[2].value = 0.333;
        $("#kappaScale").val(0.333);
        var massStar = 2.1;
        settingsId[3].value = 2.1;
        $("#starMass").val(2.1);
    }

    if (switchStar == "Procyon") {
        var teff = 6530.0;
        settingsId[0].value = 6530.0;
        $("#Teff").val(6530.0);
        var logg = 4.0;
        settingsId[1].value = 4.0;
        $("#logg").val(4.0);
        var kappaScale = 1.0;
        settingsId[2].value = 1.0;
        $("#kappaScale").val(1.0);
        var massStar = 1.4;
        settingsId[3].value = 1.4;
        $("#starMass").val(1.4);
    }



    var switchLine = "None";
    var numPreLines = 3;
    //var numPreLines = 0;


    // None: (default)
    if (settingsId[17].checked) {
        switchLine = settingsId[17].value; // radio 
    }
    /***  H does not work well - super high no densities required - seems to mess up ionization equilibrium
     // H alpha
     if (settingsId[18].checked) {
     switchLine = settingsId[18].value; // radio 
     }
     // H beta
     if (settingsId[19].checked) {
     switchLine = settingsId[19].value; // radio 
     }
     ***/

    /*
     // NaI D_1
     if (settingsId[18].checked) {
     switchLine = settingsId[18].value; // radio 
     }
     // NaI D_2
     if (settingsId[19].checked) {
     switchLine = settingsId[19].value; // radio 
     }
     
     // MgI b_1
     if (settingsId[20].checked) {
     switchLine = settingsId[20].value; // radio 
     }
     */

    //JQuery:
    // NaI D_1
    if ($("#NaID1").is(":checked")) {
        switchLine = $("#NaID1").val(); // radio        
    }
    // NaI D_2
    if ($("#NaID2").is(":checked")) {
        switchLine = $("#NaID2").val(); // radio 
    }

    // MgI b_1
    if ($("#MgIb1").is(":checked")) {
        switchLine = $("#MgIb1").val(); // radio 
    }

    /***  H does not work well - super high no densities required - seems to mess up ionization equilibrium
     if (switchLine == "Halpha") {
     var lam0 = 656.271;
     settingsId[4].value = 656.271;
     var logN = 14.5;    // initial guess
     settingsId[5].value = 14.5; // initial guess
     var logF = -0.1573;
     settingsId[6].value = -0.1573;
     var chiI = 13.6;
     settingsId[7].value = 13.6;
     var chiL = 10.2;
     settingsId[8].value = 10.2;
     var mass = 1.0;
     settingsId[10].value = 1.0;
     var logGammaCol = 1.0;
     settingsId[11].value = 1.0;        
     }
     
     if (switchLine == "Hbeta") {
     var lam0 = 486.128;
     settingsId[4].value = 486.128;
     var logN = 14.5;    // initial guess
     settingsId[5].value = 14.5; // initial guess
     var logF = -0.914;
     settingsId[6].value = -0.914;
     var chiI = 13.6;
     settingsId[7].value = 13.6;
     var chiL = 10.2;
     settingsId[8].value = 10.2;
     var mass = 1.0;
     settingsId[10].value = 1.0;
     var logGammaCol = 1.0;
     settingsId[11].value = 1.0;        
     }
     ***/

    /*
     if (switchLine == "NaID1") {
     var lam0 = 589.592;
     settingsId[4].value = 589.592;
     // var logN = 11.5;    // initial guess
     // settingsId[5].value = 11.5; // initial guess
     var A12 = 6.3;    // Grevesse & Sauval 98
     settingsId[5].value = 6.3;
     var logF = -0.495;
     settingsId[6].value = -0.495;
     var chiI = 5.139;
     settingsId[7].value = 5.139;
     var chiL = 0.0;
     settingsId[8].value = 0.0;
     var mass = 22.990;
     settingsId[10].value = 22.990;
     var logGammaCol = 1.0;
     settingsId[11].value = 1.0;
     }
     
     if (switchLine == "NaID2") {
     var lam0 = 588.995;
     settingsId[4].value = 588.995;
     //var logN = 11.5;    // initial guess
     //settingsId[5].value = 11.5; // initial guess
     var A12 = 6.3;    // Grevesse & Sauval 98
     settingsId[5].value = 6.3;
     var logF = -0.193;
     settingsId[6].value = -0.193;
     var chiI = 5.139;
     settingsId[7].value = 5.139;
     var chiL = 0.0;
     settingsId[8].value = 0.0;
     var mass = 22.990;
     settingsId[10].value = 22.990;
     var logGammaCol = 1.0;
     settingsId[11].value = 1.0;
     }
     
     if (switchLine == "MgIb1") {
     var lam0 = 518.360;
     settingsId[4].value = 518.360;
     //var logN = 13.0;    // initial guess
     //settingsId[5].value = 13.0; // initial guess
     var A12 = 7.5;    // Grevesse & Sauval 98
     settingsId[5].value = 7.5;
     var logF = -0.867;
     settingsId[6].value = -0.867;
     var chiI = 7.646;
     settingsId[7].value = 7.646;
     var chiL = 2.717;
     settingsId[8].value = 2.717;
     var mass = 24.305;
     settingsId[10].value = 24.305;
     var logGammaCol = 1.0;
     settingsId[11].value = 1.0;
     }
     */

    //JQuery:

    if (switchLine == "NaID1") {
        var lam0 = 589.592;
        settingsId[4].value = 589.592;
        $("#lambda_0").val(589.592);
        // var logN = 11.5;    // initial guess
        // settingsId[5].value = 11.5; // initial guess
        var A12 = 6.3;    // Grevesse & Sauval 98
        settingsId[5].value = 6.3;
        $("#A12").val(6.3);
        var logF = -0.495;
        settingsId[6].value = -0.495;
        $("#logf").val(-0.495);
        var chiI = 5.139;
        settingsId[7].value = 5.139;
        $("#chi_I").val(5.139);
        var chiL = 0.0;
        settingsId[8].value = 0.0;
        $("#chi_L").val(0.0);
        var mass = 22.990;
        settingsId[10].value = 22.990;
        $("#mass").val(22.990);
        var logGammaCol = 1.0;
        settingsId[11].value = 1.0;
        $("#gammaCol").val(1.0);
    }

    if (switchLine == "NaID2") {
        var lam0 = 588.995;
        settingsId[4].value = 588.995;
        $("#lambda_0").val(588.995);
        //var logN = 11.5;    // initial guess
        //settingsId[5].value = 11.5; // initial guess
        var A12 = 6.3;    // Grevesse & Sauval 98
        settingsId[5].value = 6.3;
        $("#A12").val(6.3);
        var logF = -0.193;
        settingsId[6].value = -0.193;
        $("#logf").val(-0.193);
        var chiI = 5.139;
        settingsId[7].value = 5.139;
        $("#chi_I").val(5.139);
        var chiL = 0.0;
        settingsId[8].value = 0.0;
        $("#chi_L").val(0.0);
        var mass = 22.990;
        settingsId[10].value = 22.990;
        $("#mass").val(22.990);
        var logGammaCol = 1.0;
        settingsId[11].value = 1.0;
        $("#gammaCol").val(1.0);
    }

    if (switchLine == "MgIb1") {
        var lam0 = 518.360;
        settingsId[4].value = 518.360;
        $("#lambda_0").val(518.360);
        //var logN = 13.0;    // initial guess
        //settingsId[5].value = 13.0; // initial guess
        var A12 = 7.6;    // Grevesse & Sauval 98
        settingsId[5].value = 7.6;
        $("#A12").val(7.6);
        var logF = -0.867;
        settingsId[6].value = -0.867;
        $("#logf").val(-0.867);
        var chiI = 7.646;
        settingsId[7].value = 7.646;
        $("#chi_I").val(7.646);
        var chiL = 2.717;
        settingsId[8].value = 2.717;
        $("#chi_L").val(2.717);
        var mass = 24.305;
        settingsId[10].value = 24.305;
        $("#mass").val(24.305);
        var logGammaCol = 1.0;
        settingsId[11].value = 1.0;
        $("#gammaCol").val(1.0);
    }



    var numInputs = settingsId.length;
    // Sanity check:

    var flagArr = new Array(numInputs);

    flagArr[0] = false;
    var minTeff = 3000.0;
    var maxTeff = 25000.0;
    if (teff < minTeff) {
        flagArr[0] = true;
        teff = minTeff;
        var teffStr = "3000";
        settingsId[0].value = 3000.0;
        $("#Teff").val(3000.0);
    }
    if (teff > maxTeff) {
        flagArr[0] = true;
        teff = maxTeff;
        var teffStr = "25000";
        settingsId[0].value = 25000.0;
        $("#Teff").val(25000.0);
    }
    flagArr[1] = false;
    if (logg < 1.5) {
        flagArr[1] = true;
        logg = 1.5;
        var loggStr = "1.5";
        settingsId[1].value = 1.5;
        $("#logg").val(1.5);
    }
    if (logg > 5.5) {
        flagArr[1] = true;
        logg = 5.5;
        var loggStr = "5.5";
        settingsId[1].value = 5.5;
        $("#logg").val(5.5);
    }
    flagArr[2] = false;
    if (kappaScale < 0.1) {
        flagArr[2] = true;
        kappaScale = 0.1;
        var kappaStr = "0.1";
        settingsId[2].value = 0.1;
        $("#kappaScale").val(0.1);
    }
    if (kappaScale > 3.0) {
        flagArr[2] = true;
        kappaScale = 3.0;
        var kappaStr = "3.0";
        settingsId[2].value = 3.0;
        $("#kappaScale").val(3.0);
    }
    flagArr[3] = false;
    if (massStar < 0.25) {
        flagArr[3] = true;
        massStar = 0.25;
        var massStarStr = "0.25";
        settingsId[3].value = 0.25;
        $("#starMass").val(0.25);
    }
    if (massStar > 5.0) {
        flagArr[3] = true;
        massStar = 5.0;
        var massStarStr = "5.0";
        settingsId[3].value = 10.0;
        $("#starMass").val(10.0);
    }

    var grav = Math.pow(10.0, logg);

    flagArr[4] = false;
    if (lam0 < 200.0) {
        flagArr[4] = true;
        lam0 = 200.0;
        var lamStr = "200";
        settingsId[4].value = 200.0;
        $("#lambda_0").val(200.0);
    }
    if (lam0 > 20000.0) {
        flagArr[4] = true;
        lam0 = 20000.0;
        var lamStr = "20000";
        settingsId[4].value = 20000.0;
        $("#lambda_0").val(20000.0);
    }
    flagArr[5] = false;
    if (A12 < 3.0) {
        flagArr[5] = true;
        A12 = 3.0;
        var nStr = "3.0";
        settingsId[5].value = 3.0;
        $("#A12").val(3.0);
    }
    if (A12 > 9.0) {
        flagArr[5] = true;
        A12 = 9.0;
        var nStr = "9.0";
        settingsId[5].value = 9.0;
        $("#A12").val(9.0);
    }

    var logNH = 17.0;
    var logN = (A12 - 12) + logNH;

    flagArr[6] = false;
    if (logF < -6.0) {
        flagArr[6] = true;
        logF = -6.0;
        var fStr = "-6.0";
        settingsId[6].value = -6.0;
        $("#logf").val(-6.0);
    }
    if (logF > 1.0) {
        flagArr[6] = true;
        logF = 1.0;
        var fStr = "1.0";
        settingsId[6].value = 1.0;
        $("#logf").val(1.0);
    }
    flagArr[7] = false;
    if (chiI < 5.0) {
        flagArr[7] = true;
        chiI = 5.0;
        var ionStr = "5.0";
        settingsId[7].value = 5.0;
        $("#chi_I").val(5.0);
    }
    if (chiI > 15.0) {
        flagArr[7] = true;
        chiI = 15.0;
        var ionStr = "15.0";
        settingsId[7].value = 15.0;
        $("#chi_I").val(15.0);
    }
    var ionized = false;
    // Note: Upper limit of chiL depends on value of chiI above!
    flagArr[8] = false;
    if (chiL < 0.0) {
        flagArr[8] = true;
        chiL = 0.0; // Ground state case!
        var excStr = "0.0";
        settingsId[8].value = 0.0;
        $("#chi_L").val(0.0);
    }
// choice of neutral or singly ionized stage
    if (chiL < chiI) {
        var ionized = false;
    }
    if (chiL == chiI) {

        flagArr[8] = true;
        ionized = false;
        chiL = 0.9 * chiI;
        var excStr = ionStr;
        settingsId[8].value = chiL;
        $("#chi_L").val(chiL);
    }
    if (chiL > chiI) {
        ionized = true;
        // chiL = chiL - chiI;  // No!  Not here!
        if (chiL > chiI + 5.0) {
            flagArr[8] = true;
            chiL = chiI + 5.0;
            var excStr = " " + chiL;
            settingsId[8].value = chiL;
            $("#chi_L").val(chiL);
        }
    }
    flagArr[9] = false;
    if (xiT < 0.0) {
        flagArr[9] = true;
        xiT = 0.0;
        var xitStr = "0.0";
        settingsId[9].value = 0.0;
        $("#xi_T").val(0.0);
    }
    if (xiT > 4.0) {
        flagArr[9] = true;
        xiT = 4.0;
        var xitStr = "4.0";
        settingsId[9].value = 4.0;
        $("#xi_T").val(4.0);
    }
    flagArr[10] = false;
    if (mass < 1.0) {
        flagArr[10] = true;
        mass = 1.0;
        var massStr = "1.0";
        settingsId[10].value = 1.0;
        $("#mass").val(1.0);
    }
    if (mass > 60.0) {
        flagArr[10] = true;
        mass = 60.0;
        var massStr = "60";
        settingsId[10].value = 60.0;
        $("#mass").val(60.0);
    }
    flagArr[11] = false;
    if (logGammaCol < 0.0) {
        flagArr[11] = true;
        logGammaCol = 0.0;
        var gamStr = "0.0";
        settingsId[11].value = 0.0;
        $("#gammaCol").val(0.0);
    }
    if (logGammaCol > 1.0) {
        flagArr[11] = true;
        logGammaCol = 1.0;
        var gamStr = "1.0";
        settingsId[11].value = 1.0;
        $("#gammaCol").val(1.0);
    }



    //var ionized = false; // DEBUG


    // This has to be up here for some reason:
    // Get the ID of the container div:

    var textId = document.getElementById("outPanel"); // text output area

    //var masterId = document.getElementById("container"); // graphical output area
    var plotOneId = document.getElementById("plotOne");
    var plotTwoId = document.getElementById("plotTwo");
    var plotThreeId = document.getElementById("plotThree");
    var plotFourId = document.getElementById("plotFour");
    var plotFiveId = document.getElementById("plotFive");
    var plotSixId = document.getElementById("plotSix");
    var plotSevenId = document.getElementById("plotSeven");
    var plotEightId = document.getElementById("plotEight");
    var plotNineId = document.getElementById("plotNine");



    // Begin compute code:


    //Gray structure and Voigt line code code begins here:
    // Initial set-up:
    var numDeps = 50;
    //var numMus = 10; // Guess

    var numLams = 20;
    var lamSetup = [200., 2000., numLams]; //Start, end wavelength (nm), number of lambdas

    // Solar parameters:
    var teffSun = 5778.0;
    var loggSun = 4.44;
    var gravSun = Math.pow(10.0, loggSun);
    var kappaScaleSun = 1.0;
    var logRadius = 0.5 * (Math.log(massStar) + Math.log(gravSun) - Math.log(grav));
    var radius = Math.exp(logRadius);
    //var radius = Math.sqrt(massStar * gravSun / grav); // solar radii
    var logLum = 2.0 * Math.log(radius) + 4.0 * Math.log(teff / teffSun);
    var bolLum = Math.exp(logLum); // L_Bol in solar luminosities 


    // Wavlength grid for photometry, in nm:
    //// Johnson UBVRI: - Allen's Astrophys. Quant. 4th Ed., pg. 386
    //double[] lamUBVRI = {360.0, 440.0, 550.0, 700.0, 880.0};  nm
    // Johnson UBV + Kron-Cousins RI: - Allen's Astrophys. Quant. 4th Ed., pg. 386
    //  - Row 0 is band centre wavelengths (nm)
    //  - Row 1 is half-power half-widths (nm)
    var lamUBVRI = [[360.0, 440.0, 550.0, 640.0, 790.0],
        [35.0, 50.0, 45.0, 87.5, 70.0]];
    var logE = logTen(Math.E); // for debug output

    //log_10 Rosseland optical depth scale  
    //Java: double tauRos[][] = TauScale.tauScale(numDeps);
    //var logTauRos = TauScale(numDeps);
    var tauRos = tauScale(numDeps);
    //Gray kinetic temeprature structure:
    // Java: double temp[][] = Temperature.temperature(numDeps, teff, tauRos);
    var temp = temperature(numDeps, teff, tauRos);
    //Now do the same for the Sun, for reference:
    var tempSun = temperature(numDeps, teffSun, tauRos);

    /*
     * These array declarations will be used when the atmospheric structure is
     * debugged enough to hard-wire the solar structure to save execution time:
     var tempRow0 = new Array(numDeps);
     var tempRow1 = new Array(numDeps);
     var kappRow0 = new Array(numDeps);
     var kappRow1 = new Array(numDeps);
     var pressRow0 = new Array(numDeps);
     var pressRow1 = new Array(numDeps);
     var pressRow2 = new Array(numDeps);
     var pressRow3 = new Array(numDeps);
     var rhoRow0 = new Array(numDeps);
     var rhoRow1 = new Array(numDeps);
     
     var tempSun = [tempRow0, tempRow1];
     var kappaSun = [kappRow0, kappRow1];
     var pressSun = [pressRow0, pressRow1, pressRow2, pressRow3];
     var rhoSun = [rhoRow0, rhoRow1];
     var depthsSun = new Array(numDeps);
     */


    //
// Create kappa structure here: Initialize solar kappa_Ross structure and
    // scale it by kappaScale:
    var kappa = kappas(numDeps, kappaScale, tauRos, temp, tempSun, logg, loggSun, teff);
    var kappaSun = kappas(numDeps, kappaScaleSun, tauRos, tempSun, tempSun, loggSun, loggSun, teffSun);
    //Next solve hydrostatic eq for P scale on the tau scale - need to pick a depth dependent kappa value!
    //   - scale kapp_Ross with log(g) from solar value? - Kramers opacity law?
    //   - dP/dTau = g/kappa
    //press is a 4 x numDeps array:
    // rows 0 & 1 are linear and log *gas* pressure, respectively
    // rows 2 & 3 are linear and log *radiation* pressure
    var press = hydrostatic(numDeps, grav, tauRos, kappa, temp);
    // Then solve eos for the rho scale - need to pick a mean molecular weight, mu
    var rho = state(numDeps, temp, press);
    // Then construct geometric depth scale from tau, kappa and rho
    var depths = depthScale(numDeps, tauRos, kappa, rho);
    //Now do the same for the Sun, for reference:

    var pressSun = hydrostatic(numDeps, gravSun, tauRos, kappaSun, tempSun);
    var rhoSun = state(numDeps, tempSun, pressSun);
    // var depthsSun = depthScale(numDeps, tauRos, kappaSun, rhoSun);
    // Limb darkening:
    // Establish wavelength grid:
    var lambdaScale = lamgrid(numLams, lamSetup);
    // Set up theta grid
    //  cosTheta is a 2xnumMus array:
    // row 0 is used for Gaussian quadrature weights
    // row 1 is used for cos(theta) values
    // Gaussian quadrature:
    // Number of angles, numMus, will have to be determined after the fact
    var cosTheta = thetas();
    var numThetas = cosTheta[0].length;
    // Solve formal sol of rad trans eq for outgoing surfaace I(0, theta)


    var intens = new Array(numLams);
    for (var row = 0; row < numLams; row++) {
        intens[row] = new Array(numThetas);
    }

    var intensLam = new Array(numThetas);
    var dum0 = [];
    var dum1 = [];
    dum0.length = numLams;
    dum1.lenght = numLams;
    var flux = [dum0, dum1];
    var fluxLam = new Array(2);
    var lambda1, lambda2, fluxSurfBol, logFluxSurfBol;
    fluxSurfBol = 0;
    var lineMode;
    lineMode = false;

    for (var il = 0; il < numLams; il++) {


        intensLam = formalSoln(numDeps,
                cosTheta, lambdaScale[il], tauRos, temp, lineMode);
// For some weird reason, JavaScript does not like the function name flux()!!! - Change to flux2()
        fluxLam = flux2(intensLam, cosTheta);
        for (var it = 0; it < numThetas; it++) {
            intens[il][it] = intensLam[it];
        } //it loop - thetas

        flux[0][il] = fluxLam[0];
        flux[1][il] = fluxLam[1];

        //Teff test
        //if (il > 1) {
        //    lambda2 = lambdaScale[il] * 1.0E-7; // convert nm to cm
        //    lambda1 = lambdaScale[il - 1] * 1.0E-7; // convert nm to cm
        //    fluxSurfBol = fluxSurfBol
        //            + fluxLam[0] * (lambda2 - lambda1);
        //}



    } //il loop - lambdas


    var sigma = 5.670373E-5; //Stefan-Boltzmann constant ergs/s/cm^2/K^4  
    var logSigma = Math.log(sigma);
    // Teff test
    //logFluxSurfBol = Math.log(fluxSurfBol);
    //var logTeffFlux = (logFluxSurfBol - logSigma) / 4.0;
    //var teffFlux = Math.exp(logTeffFlux);
    //var testInter = interpol(lambdaScale, flux, 500.0);
    //
    //Compute JOhnson-Cousins photometric color indices:
    var colors = UBVRI(lamUBVRI, lambdaScale, flux, numDeps, tauRos, temp);
    // Line profile section:

    // Set up grid of lambda points sampling red wing and core of half-line profile:


    var linePoints = lineGrid(lam0, mass, xiT, numDeps, teff);
    //Compute area-normalized depth-independent line profile "phi_lambda(lambda)"
    //public static double[][] voigt(double lam0In, double massIn, double logGammaCol, double xiTIn,
    //    int numDeps, double[][] tauRos, double teff, double[][] press, double[][] pressSun) {
    var lineProf = voigt(linePoints, lam0, logGammaCol,
            numDeps, teff, tauRos, temp, press, tempSun, pressSun);
    //Compute depth-dependent logarithmic monochromatic extinction co-efficient, kappa_lambda(lambda, tauRos):
    //public static double[][] lineKap(double lam0In, double logNlIn, double logFluIn, double[][] lineProf,
    //    int numDeps, double kappaScale, double[][] tauRos, double[][] temp, double[][] rho) {


    // var logKappaL = lineKap(lam0, logN, logF, ionized, chiI, chiL, linePoints, lineProf,
    //         numDeps, kappaScale, tauRos, temp, rho);
    //         
    // Level population now computed in LevelPops.levelPops()


    var logNums = levelPops(lam0, logN, ionized, chiI, chiL, linePoints, lineProf,
            numDeps, kappaScale, tauRos, temp, rho);
    var logKappaL = lineKap(lam0, logNums, logF, linePoints, lineProf,
            numDeps, kappaScale, tauRos, temp, rho);
    //public static double[][] tauLambda(int numDeps, double[][] lineProf, double[][] logKappaL, 
    //                             double[][] kappa, double[][] tauRos, double[][] rho, double[] depths) {
    //Compute monochromatic optical depth scale, Tau_lambda throughout line profile
    //CAUTION: Returns numPoints+1 x numDeps array: the numPoints+1st row holds the line centre continuum tau scale
    // Method 1: double logTauL[][] = LineTau.tauLambda(numDeps, lineProf, logKappaL,
    // Method 1:        kappa, tauRos, rho, depths);
    // Method 2:
    var logTauL = tauLambda(numDeps, linePoints, logKappaL,
            kappa, tauRos, rhoSun);
    //Evaluate formal solution of rad trans eq at each lambda throughout line profile
    // Initial set to put lambda and tau arrays into form that formalsoln expects
    var numPoints = linePoints[0].length + 1; //Extra wavelength point at end for monochromatic continuum tau scale

    var lineLambdas = new Array(numPoints);
    var lineIntens = new Array(numPoints);
    for (var row = 0; row < numPoints; row++) {
        lineIntens[row] = new Array(numThetas);
    }
//Java: double[][] lineIntens = new double[numPoints][numThetas];

    var lineIntensLam = new Array(numThetas);
    var dummy11 = new Array(numPoints);
    var dummy22 = new Array(numPoints);
    var lineFlux = [dummy11, dummy22];
    var lineFluxLam = new Array(2);
    var dummy111 = new Array(numDeps);
    var dummy222 = new Array(numDeps);
    var thisTau = [dummy111, dummy222];
    lineMode = true;
    for (var il = 0; il < numPoints; il++) {


        if (il === numPoints - 1) {
            lineLambdas[il] = lam0; // Extra row for line centre continuum taus scale
        } else {
            lineLambdas[il] = (1.0E7 * linePoints[0][il]) + lam0; //convert back to nm
        }

        for (var id = 0; id < numDeps; id++) {
            thisTau[1][id] = logTauL[il][id];
            thisTau[0][id] = Math.exp(logTauL[il][id]);
        } // id loop


        lineIntensLam = formalSoln(numDeps,
                cosTheta, lineLambdas[il], thisTau, temp, lineMode);
        lineFluxLam = flux2(lineIntensLam, cosTheta);
        for (var it = 0; it < numThetas; it++) {
            lineIntens[il][it] = lineIntensLam[it];
        } //it loop - thetas

        lineFlux[0][il] = lineFluxLam[0];
        lineFlux[1][il] = lineFluxLam[1];
    } //il loop



//Get equivalent width, W_lambda, in pm - picometers:
    var Wlambda = eqWidth(lineFlux, linePoints, lam0);
// *********************



// Text output section:

//    
// Set up the canvas:
//

    // **********  Basic canvas parameters: These are numbers in px - needed for calculations:
    // All plots and other output must fit within this region to be white-washed between runs

    var xRangeT = 1550;
    var yRangeT = 65;
    var xOffsetT = 10;
    var yOffsetT = 10;
    var charToPxT = 4; // width of typical character font in pixels - CAUTION: finesse!

    var zeroInt = 0;


    //these are the corresponding strings ready to be assigned to HTML style attributes


    var xRangeTStr = numToPxStrng(xRangeT);
    var yRangeTStr = numToPxStrng(yRangeT);
    var xOffsetTStr = numToPxStrng(xOffsetT);
    var yOffsetTStr = numToPxStrng(yOffsetT);
    // Very first thing on each load: White-wash the canvas!!

    var washTId = document.createElement("div");
    var washTWidth = xRangeT + xOffsetT;
    var washTHeight = yRangeT + yOffsetT;
    var washTTop = yOffsetT;
    var washTWidthStr = numToPxStrng(washTWidth);
    var washTHeightStr = numToPxStrng(washTHeight);
    var washTTopStr = numToPxStrng(washTTop);
    washTId.id = "washT";
    washTId.style.position = "absolute";
    washTId.style.width = washTWidthStr;
    washTId.style.height = washTHeightStr;
    washTId.style.marginTop = washTTopStr;
    washTId.style.marginLeft = "0px";
    washTId.style.opacity = 1.0;
    washTId.style.backgroundColor = "#EEEEEE";
    //washId.style.zIndex = -1;
    washTId.style.zIndex = 0;
    //washTId.style.border = "2px blue solid";

    //Wash the canvas:
    textId.appendChild(washTId);
    var roundNum, remain;

    // R & L_Bol:
    var colr = 0;
    var xTab = 60;
    roundNum = radius.toPrecision(3);
    txtPrint("<span title='Stellar radius'><em>R</em> = </span> "
            + roundNum
            + " <span title='Solar radii'><a href='http://en.wikipedia.org/wiki/Solar_radius' target='_blank'><em>R</em><sub>Sun</sub></a></span> "
            , 20 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId);

    //numPrint(roundNum, 60 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId); //debug  
    //txtPrint("<span title='Solar radii'><a href='http://en.wikipedia.org/wiki/Solar_radius' target='_blank'><em>R</em><sub>Sun</sub></a></span> ", 110 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId);
    roundNum = bolLum.toPrecision(3);
    txtPrint("<span title='Bolometric luminosity'><a href='http://en.wikipedia.org/wiki/Luminosity' target='_blank'><em>L</em><sub>Bol</sub></a> = </span> "
            + roundNum
            + " <span title='Solar luminosities'><a href='http://en.wikipedia.org/wiki/Solar_luminosity' target='_blank'><em>L</em><sub>Sun</sub></a></span> ", 20 + colr * xTab, 40, zeroInt, zeroInt, zeroInt, textId);

    //numPrint(roundNum, 60 + colr * xTab, 40, zeroInt, zeroInt, zeroInt, textId); //debug  
    //txtPrint("<span title='Solar luminosities'><a href='http://en.wikipedia.org/wiki/Solar_luminosity' target='_blank'><em>L</em><sub>Sun</sub></a></span> ", 110 + colr * xTab, 40, zeroInt, zeroInt, zeroInt, textId);
// 
// Equivalent width:
    roundNum = Wlambda.toFixed(2);
    txtPrint("<span title='Equivalent width: A measure of spectral line strength'>Spectral line <a href='http://en.wikipedia.org/wiki/Equivalent_width' target='_blank'>W<sub>&#955</sub></a>: </span>"
            + roundNum
            + " <span title='picometers'><a href='http://en.wikipedia.org/wiki/Picometre' target='_blank'>pm</a></span>", 180, 40, zeroInt, zeroInt, zeroInt, textId);
    ////remain = (Wlambda * 1000.0) % 10;
    ////roundNum = (Wlambda) - (remain / 1000.0);

    //roundNum = Wlambda.toFixed(2);
    //numPrint(roundNum, 330, 40, zeroInt, zeroInt, zeroInt, textId); //debug
    //txtPrint("<span title='picometers'><a href='http://en.wikipedia.org/wiki/Picometre' target='_blank'>pm</a></span>", 370, 40, zeroInt, zeroInt, zeroInt, textId);

    // UBVRI indices
    var xTab = 80;
    var colr = 0;
    var roundNum0 = colors[0].toFixed(2);
    var roundNum1 = colors[1].toFixed(2);
    var roundNum2 = colors[2].toFixed(2);
    var roundNum3 = colors[3].toFixed(2);
    var roundNum4 = colors[4].toFixed(2);

    txtPrint("<a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins U-B photometric color index' target='_blank'><span style='color:purple'>U</span>-" +
            "<span style='color:blue'>B</span></a>: " + roundNum0
            + " <a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins B-V photometric color index' target='_blank'><span style='color:blue'>B</span>-" +
            "<span style='color:#00FF88'>V</span></a>: " + roundNum1
            + " <a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins V-R photometric color index' target='_blank'><span style='color:#00FF88'>V</span>-" +
            "<span style='color:red'>R</span></a>: " + roundNum2
            + " <a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins V-I photometric color index' target='_blank'><span style='color:#00FF88'>V</span>-" +
            "<span style='color:red'>I</span></a>: " + roundNum3
            + " <a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins R-I photometric color index' target='_blank'><span style='color:red'>R</span>-" +
            "<span style='color:brown'>I</span></a>: " + roundNum4, 180 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId);

    /*
     txtPrint("<a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins U-B photometric color index' target='_blank'><span style='color:purple'>U</span>" + "-" +
     "<span style='color:blue'>B</span></a>" + ": ", 180 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId);
     //remain = (colors[0] * 1000.0) % 10;
     //roundNum = colors[0] - (remain / 1000.0);
     numPrint(roundNum, 220 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId); //debug  
     colr = 1;
     txtPrint("<a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins B-V photometric color index' target='_blank'><span style='color:blue'>B</span>" + "-" +
     "<span style='color:#00FF88'>V</span></a>" + ": ", 180 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId);
     //remain = (colors[1] * 1000.0) % 10;
     //roundNum = colors[1] - (remain / 1000.0);
     roundNum = colors[1].toFixed(2);
     numPrint(roundNum, 220 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId); //debug  
     colr = 2;
     txtPrint("<a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins V-R photometric color index' target='_blank'><span style='color:#00FF88'>V</span>" + "-" +
     "<span style='color:red'>R</span></a>" + ": ", 180 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId);
     //remain = (colors[2] * 1000.0) % 10;
     //roundNum = colors[2] - (remain / 1000.0);
     roundNum = colors[2].toFixed(2);
     numPrint(roundNum, 220 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId); //debug  
     colr = 3;
     txtPrint("<a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins V-I photometric color index' target='_blank'><span style='color:#00FF88'>V</span>" + "-" +
     "<span style='color:red'>I</span></a>" + ": ", 180 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId);
     //remain = (colors[3] * 1000.0) % 10;
     //roundNum = colors[3] - (remain / 1000.0);
     roundNum = colors[3].toFixed(2);
     numPrint(roundNum, 220 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId); //debug  
     colr = 4;
     txtPrint("<a href='http://en.wikipedia.org/wiki/UBV_photometric_system' title='Johnson-Cousins R-I photometric color index' target='_blank'><span style='color:red'>R</span>" + "-" +
     "<span style='color:brown'>I</span></a>" + ": ", 180 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId);
     //remain = (colors[4] * 1000.0) % 10;
     //roundNum = colors[4] - (remain / 1000.0);
     roundNum = colors[4].toFixed(2);
     numPrint(roundNum, 220 + colr * xTab, 15, zeroInt, zeroInt, zeroInt, textId); //debug
     */



    // Echo back the *actual* input parameters:

    xTab = 140;
    var outString, fullNum, j;
    var numReportable = numInputs - numPreStars - numPreLines - 1;

    var echoText = "<table><tr>";
    for (var i = 0; i < numReportable - 1; i++) {
        var fullNum = settingsId[i].value;
        //roundNum = fullNum.toPrecision(2);
        roundNum = fullNum;
        if (flagArr[i]) {
            outString = "<td>   <span style='color:red'>   " + settingsId[i].name + "</br>" + roundNum.toString(10) + "   </span>   </td>";
        } else {
            outString = "<td>   <span style='color:black'>   " + settingsId[i].name + "</br>" + roundNum.toString(10) + "   </span>   </td>";
        }
        //if (i == numReportable / 2){
        //    echoText = echoText + "</tr><tr>";  // New row
        //};
        echoText = echoText + "   " + outString + "   ";
    }  // i loop
    echoText = echoText + "</tr></table>";
    txtPrint(echoText, 600, 10, zeroInt, zeroInt, zeroInt, textId);

    /*
     for (var i = 0; i < numReportable - 1; i++) {
     var fullNum = settingsId[i].value;
     //roundNum = fullNum.toPrecision(2);
     roundNum = fullNum;
     if (flagArr[i]) {
     outString = "<span style='color:red'>" + settingsId[i].name + ": " + roundNum.toString(10) + "</span>";
     } else {
     outString = "<span style='color:black'>" + settingsId[i].name + ": " + roundNum.toString(10) + "</span>";
     }
     
     
     if (i <= numReportable / 2) {
     //            txtPrint(settingsId[i].name + ": ", 620 + i * xTab, 15, zeroInt, zeroInt, zeroInt, textId);
     //            //roundNum = fullNum.toFixed(2);  //Doesn't work !?        
     //            numPrint(fullNum, 700 + i * xTab, 15, , zeroInt, zeroInt, zeroInt, textId);
     txtPrint(outString, 620 + i * xTab, 15, zeroInt, zeroInt, zeroInt, textId);
     } else {
     j = i - numReportable / 2;
     //            txtPrint(settingsId[i].name + ": ", 620 + j * xTab, 40, zeroInt, zeroInt, zeroInt, textId);
     //            //roundNum = fullNum.toFixed(2);  //Doesn't work !?        
     //            numPrint(fullNum, 700 + j * xTab, 40, zeroInt, zeroInt, zeroInt, textId);
     txtPrint(outString, 620 + j * xTab, 40, zeroInt, zeroInt, zeroInt, textId);
     }
     
     }  // i loop
     */

// Graphical output section:


//  NOTE!!!
//  
//  The remaining 3000+ lines of code or so are all devoted to the graphical output plots.  
//  Have not been able to spin this stuff off into separare function that can be called from 
//  seperate source files by the HTML code.  This is baaaaad!  :-(
//    
// Each of these plots takes ~400 lines of code to convert JavaScript calculations into 
// HTML elements!!
// I'd like to spin each plot into a separate function in its own *.js source file, but it doesn't 
// seem to work - !!???

// ***** WARNING: Do NOT rearrange order of plot-wise code-blocks.  Some blocks use variables decalred and initialized in
// previous blocks.  If you want to re-arrange the plots in the WWW page, change the row and column number
// indices (plotRow, plotCol) at the beginning of each block




    var colorTemp = function(temp, minTeff, maxTeff, dark) {

        // Converts a gas temperature in K to a approximate rendering RGB color according
        // to the Wien displacement law of blackbody radiation

        var wien = 2.8977721E-1; // Wien's displacement law constant in cm K
        var logMaxPeak = Math.log(wien / minTeff);
        var logMinPeak = Math.log(wien / maxTeff);
        //var greenTeff = 6000.; // K
        var tempSun = 6000;
        //var logGreenPeak = Math.log(wien / greenTeff);
//
        //var logLamPeak = Math.log(wien / temp); // lambda in cm
        var tempVega = 9550.0;

        // Color palate - try keeping the amoung of green neutral and varying the red and blue inversely with Teff
// For hex RRGGBB format there are 16^2 = 256 different values for each of R, G, and B:
// compute how much red and blue should be in the render colour:
// Do not usef full range - buffer by 10 at either end to avoid near- black and white values - ??

        //var gg = 0.0;
        var rr, gg, bb;
        // Red side of green peak
        // if (logLamPeak >= logGreenPeak) {
        //   gg = 0.5 * Math.pow(16, 2) * (logMaxPeak - logLamPeak) / (logMaxPeak - logGreenPeak);
        //    // blue side of green peak
        // } else {
        //    gg = 0.5 * Math.pow(16, 2) * (logLamPeak - logMinPeak) / (logGreenPeak - logMinPeak);
        // }
        if (temp <= tempSun) {
            rr = 255.0;
            gg = 255.0 * (1.0 - (tempSun - temp) / (tempSun - minTeff));
            bb = 255.0 * (1.0 - (tempVega - temp) / (tempVega - minTeff));
        } else if (temp <= tempVega && temp >= tempSun) {
            rr = 255.0;
            gg = 255.0;
            bb = 255.0 * (1.0 - (tempVega - temp) / (tempVega - minTeff));
        } else {
            rr = 255.0 * (1.0 - (temp - tempVega) / (maxTeff - tempVega));
            gg = 255.0 * (1.0 - (temp - tempVega) / (maxTeff - tempVega));
            bb = 255.0;
        }

        var ggI = Math.ceil(dark * gg);
////var tempDisk2 = teff;
// The higher the Teff, the more blue... :
//        //var bb = Math.pow(16, 2) * (logTen(tempDisk2) - logTen(minTeff)) / (logTen(maxTeff) - logTen(minTeff));  // logarithmically 
        //       //var bb = Math.pow(16, 2) - Math.pow(16, 2) * (logLamPeak - logMinPeak) / (logMaxPeak - logMinPeak); 
        //       var bb = Math.pow(16, 2) * (logMaxPeak - logLamPeak) / (logMaxPeak - logMinPeak);
////    var bb = Math.pow(16,2) * ( msTeffs[k] - minTeff )/( maxTeff - minTeff );  // linearly
        var bbI = Math.floor(dark * bb); //MUST convert to integer

//// ... and the less red:
        //       //var rr = Math.pow(16, 2) - Math.pow(16, 2) * (logTen(tempDisk2) - logTen(minTeff)) / (logTen(maxTeff) - logTen(minTeff)); //logarithmically       
        //       var rr = Math.pow(16, 2) * Math.pow((logLamPeak - logMinPeak) / (logMaxPeak - logMinPeak), 2);
////    var rr =  Math.pow(16,2) - Math.pow(16,2) * ( msTeffs[k] - minTeff )/( maxTeff - minTeff ); //linearly
        var rrI = Math.floor(dark * rr); //MUST convert to integer

        //var starRGBHex = "rgb(" + rrI + "," + ggI + "," + bbI + ")";
        //return starRGBHex;

        var RGBArr = new Array(3);
        RGBArr = [rrI, ggI, bbI];
        return RGBArr;

    };




//    
// Set up the canvas:
//

// **********  Basic canvas parameters: These are numbers in px - needed for calculations:
// All plots and other output must fit within this region to be white-washed between runs

    /*
     var xRange = 1850;
     var yRange = 600;
     */
    var xOffset = 10;
    var yOffset = 20 + yRangeT + yOffsetT;
    var charToPx = 4; // width of typical character font in pixels - CAUTION: finesse!


    var xRange = 300;
    var yRange = 200;
    //var xOffset = 10;
    //var yOffset = 20;

    //these are the corresponding strings ready to be assigned to HTML style attributes


    var xRangeStr = numToPxStrng(xRange);
    var yRangeStr = numToPxStrng(yRange);
    var xOffsetStr = numToPxStrng(xOffset);
    var yOffsetStr = numToPxStrng(yOffset);
    //******* tick mark, label, and axis name properties: 

    var wColor = "#F8F8F8";

    var washer = function(xOffset, yOffset, wColor, areaId) {
        // Very first thing on each load: White-wash the canvas!!

        var washXMargin = 150;
        var washYMargin = 110;
        var washWidth = xRange + washXMargin; //+ xOffset;
        var washHeight = yRange + washYMargin; //+ yOffset;
        var washTop = yOffset - washYMargin / 2 - 16;
        var washLeft = xOffset - washXMargin / 2 - 50;
        var washWidthStr = numToPxStrng(washWidth);
        var washHeightStr = numToPxStrng(washHeight);
        var washTopStr = numToPxStrng(washTop);
        var washLeftStr = numToPxStrng(washLeft);



        var washId = document.createElement("div");
        washId.id = "wash";
        washId.style.position = "absolute";
        washId.style.width = washWidthStr;
        washId.style.height = washHeightStr;
        washId.style.marginTop = washTopStr;
        //washId.style.marginLeft = "0px";
        washId.style.marginLeft = washLeftStr;
        washId.style.opacity = 1.0;
        //washId.style.backgroundColor = "#EEEEEE";
        washId.style.backgroundColor = wColor;
        //washId.style.zIndex = -1;
        washId.style.zIndex = 0;
        //washId.style.border = "1px gray solid";

        //Wash the canvas:
        //masterId.appendChild(washId);
        areaId.appendChild(washId);
    };


    // Line center, lambda_0 

    var keyLambds = halfPower(numPoints, lineFlux);


    var numDeps = depths.length;
    var xOffsetStr = numToPxStrng(xOffset);
    var yOffsetStr = numToPxStrng(yOffset);
    // ******Individual plot size parameters:


    var charToPx = 4; // width of typical character font in pixels - CAUTION: finesse!



    //******* tick mark, label, and axis name properties: 

    //tick marks:
    var tickHeight = 8;
    var tickHeightStr = numToPxStrng(tickHeight);
    var tickWidth = "0px";
    var tickColor = "black";
    var tickBorder = "1px black solid";
    //labels:
    var labelHeight = 15;
    var labelHeightStr = numToPxStrng(labelHeight);
    var labelWidth = 80;
    var labelWidthStr = numToPxStrng(labelWidth);



    var standForm = function(x) {
        // Turn any old number into the nearest number in standard form with a whole number exponent
        // and a mantissa rounded to the nearest canonical value appropriate for labeling a tick mark 

        var numParts = [2];
        var isNeg = false;

        if (x == 0.0) {
            numParts = [0.0, 0.0];
        } else {

            if (x < 0) {
                isNeg = true;
                x = -1.0 * x;
            }

            var b = logTen(x);
            var n = Math.floor(b);
            var a = x / Math.pow(10.0, n);

            if (isNeg) {
                a = -1.0 * a;
            }

            numParts[0] = a;  //mantissa
            numParts[1] = n; //exponent
        }

        return numParts;

    };

    var rounder = function(x, n, flag) {

        // Return a number rounded up or down to n decimal places (sort of?)

        var y, z;

        n = Math.abs(Math.floor(n)); //n was supposed to be a positive whole number anyway
        if (flag != "up" && flag != "down") {
            flag = "down";
        }

        if (n == 0) {
            z = x;
        } else {
            var fctr = Math.pow(10.0, n);
            var fx = 1.0 * x;
            y = fx * fctr;
            if (flag == "up") {
                z = Math.ceil(y);
            } else {
                z = Math.floor(y);
            }

            var fz = 1.0 * z;
            fz = fz / fctr;
        }

        return fz;

    };

    var XBar = function(plotRow, plotCol,
            yVal, minYDataIn, maxYDataIn,
            color, areaId) {

        var xOffset = 100 + plotCol * (xRange + 150);
        var yOffset = 100 + yRangeT + yOffsetT + plotRow * (yRange + 120);


        var xOffsetStr = numToPxStrng(xOffset);
        var yOffsetStr = numToPxStrng(yOffset);
        var xLowerYOffset = yOffset + yRange;
        var xLowerYOffsetStr = numToPxStrng(xLowerYOffset);


        //x-axis name properties:

        var yTickXOffset = xOffset - tickHeight / 2; //height and width reversed for y-ticks
        var yTickXOffsetStr = numToPxStrng(yTickXOffset);

        var barWidth = xRange;
        var barWidthStr = numToPxStrng(barWidth);
        var barHeightStr = "1px";

        var yTickPos = yRange * (yVal - minYDataIn) / (maxYDataIn - minYDataIn);
        //       xTickPos = xTickPos;


        var yShift = xLowerYOffset - yTickPos;

//stringify and add unit:
        var yShiftStr = numToPxStrng(yShift);

        //console.log("XBar: yTickPos, yShift, minYDataIn, maxYDataIn, yRange, yVal: "
        //        + yTickPos + " " + yShift + " " + minYDataIn + " " + maxYDataIn + " " + yRange + " " + yVal);

// Make the y-tick mark, Teff:

        var yTickId = document.createElement("div");
        yTickId.class = "ytick";
        yTickId.style.position = "absolute";
        yTickId.style.display = "block";
        yTickId.style.marginLeft = yTickXOffsetStr;
        //yTickId.style.border = tickBorder;
// Note that for y ticks, the height and width are reversed!:
        yTickId.style.width = barWidthStr;
        yTickId.style.height = barHeightStr;
        yTickId.style.zIndex = 0;
        yTickId.style.marginTop = yShiftStr;
        yTickId.style.backgroundColor = color;
//Append the tickmark to the axis element
        //masterId.appendChild(yTickId);
        areaId.appendChild(yTickId);

        return yShift;

    };

    var YBar = function(plotRow, plotCol,
            xVal, minXDataIn, maxXDataIn,
            color, areaId) {

        var xOffset = 100 + plotCol * (xRange + 150);
        var yOffset = 100 + yRangeT + yOffsetT + plotRow * (yRange + 120);

        var xOffsetStr = numToPxStrng(xOffset);
        var yOffsetStr = numToPxStrng(yOffset);
        var xLowerYOffset = yOffset + yRange;
        var xLowerYOffsetStr = numToPxStrng(xLowerYOffset);

        var xTickYOffset = xLowerYOffset - tickHeight / 2;
        var xTickYOffsetStr = numToPxStrng(xTickYOffset);


        var barHeight = yRange;
        var barHeightStr = numToPxStrng(barHeight);
        var barWidthStr = "1px";


        var xTickPos = xRange * (xVal - minXDataIn) / (maxXDataIn - minXDataIn);
        //       xTickPos = xTickPos;
        //console.log("YBar: xVal, minXDataIn, maxXDataIn: " + xVal + " " + minXDataIn + " " + maxXDataIn);

        var xShift = xOffset + xTickPos;

//stringify and add unit:
        var xShiftStr = numToPxStrng(xShift);
        //console.log("YBar: xTickPos, xOffset, xShift, xTickYOffsetStr, barWidthStr, barHeightStr: " +
        //        xTickPos + " " + xOffset + " " + xShift + " " + xTickYOffsetStr + " " + barWidthStr + " " + barHeightStr);

// Make the x-tick mark, Teff:

        var xTickId = document.createElement("div");
        xTickId.class = "xtick";
        xTickId.style.position = "absolute";
        xTickId.style.display = "block";
        xTickId.style.marginTop = yOffsetStr;
        //xTickId.style.marginTop = xLowerYOffsetStr;
        xTickId.style.marginLeft = xShiftStr;
        xTickId.style.height = barHeightStr;
        xTickId.style.width = barWidthStr;
        //xTickId.style.border = tickBorder;
        xTickId.style.backgroundColor = color;
        xTickId.style.zIndex = 0;
//Append the tickmark to the axis element
        //masterId.appendChild(xTickId);
        areaId.appendChild(xTickId);

        return xShift;

    };


    var XAxis = function(plotRow, plotCol,
            numXTicks, minXDataIn, maxXDataIn, xAxisName,
            areaId) {

        var axisParams = new Array(6);

        // Variables to handle normalization and rounding:
        var numParts = new Array(2);
        var mantissa;

        var xOffset = 100 + plotCol * (xRange + 150);
        var yOffset = 100 + yRangeT + yOffsetT + plotRow * (yRange + 120);

        axisParams[0] = xOffset;
        axisParams[4] = yOffset;

        var xOffsetStr = numToPxStrng(xOffset);
        var yOffsetStr = numToPxStrng(yOffset);
        var xLowerYOffset = yOffset + yRange;
        var xLowerYOffsetStr = numToPxStrng(xLowerYOffset);

        var xTickYOffset = xLowerYOffset - tickHeight / 2;
        var xTickYOffsetStr = numToPxStrng(xTickYOffset);
        var xLabelYOffset = xLowerYOffset + labelHeight;
        var xLabelYOffsetStr = numToPxStrng(xLabelYOffset);
        //x-axis name properties:
        var xNameYOffset = xLowerYOffset + 2 * labelHeight;
        //var xNameYOffsetStr = numToPxStrng(xNameYOffset);
        var xNameXOffset = Math.floor(xRange / 2) + xOffset;
        //var xNameXOffsetStr = numToPxStrng(xNameXOffset);

        axisParams[5] = xLowerYOffset;


        washer(xOffset, yOffset, wColor, areaId);

        //console.log(xOffsetStr);

        var xId = document.createElement("div");
        xId.class = "xaxis";
        xId.style.position = "absolute";
        xId.style.width = xRangeStr;
        xId.style.height = "1px";
        xId.style.marginTop = xLowerYOffsetStr;
        xId.style.marginLeft = xOffsetStr;
        xId.style.border = "1px black solid";
        //Draw the x-axis:^M
        //masterId.appendChild(xId);
        areaId.appendChild(xId);

        numParts = standForm(maxXDataIn);
        mantissa = rounder(numParts[0], 1, "up");
        maxXData = mantissa * Math.pow(10.0, numParts[1]);
        numParts = standForm(minXDataIn);
        mantissa = rounder(numParts[0], 1, "down");
        minXData = mantissa * Math.pow(10.0, numParts[1]);
        var rangeXData = maxXData - minXData;
        var deltaXData = rangeXData / (1.0 * numXTicks);
        numParts = standForm(deltaXData);
        mantissa = rounder(numParts[0], 1, "down");
        deltaXData = mantissa * Math.pow(10.0, numParts[1]);
        var deltaXPxl = xRange / numXTicks;


        axisParams[1] = rangeXData;
        axisParams[2] = deltaXData;
        axisParams[3] = deltaXPxl;

        var ii;
        for (var i = 0; i < numXTicks; i++) {

            ii = 1.0 * i;
            var xTickPos = ii * deltaXPxl;
            var xTickVal = minXData + (ii * deltaXData);
            var xTickRound = xTickVal.toPrecision(2);
            //var xTickRound = xTickVal;
            var xTickValStr = xTickRound.toString(10);
            // horizontal position in pixels - data values increase rightward:
            var xShift = xOffset + xTickPos;
//stringify and add unit:
            var xShiftStr = numToPxStrng(xShift);
// Make the x-tick mark, Teff:

            var xTickId = document.createElement("div");
            xTickId.class = "xtick";
            xTickId.style.position = "absolute";
            xTickId.style.display = "inline";
            xTickId.style.marginTop = xTickYOffsetStr;
            //xTickId.style.marginTop = xLowerYOffsetStr;
            xTickId.style.marginLeft = xShiftStr;
            xTickId.style.height = tickHeightStr;
            xTickId.style.width = tickWidth;
            xTickId.style.border = tickBorder;
            xTickId.style.zIndex = 0;
//Append the tickmark to the axis element
            //masterId.appendChild(xTickId);
            areaId.appendChild(xTickId);
            //Make the tick label, Teff:

            txtPrint("<span style='font-size:small'>" + xTickValStr + "</span>",
                    xShift, xLabelYOffset, zeroInt, zeroInt, zeroInt, areaId);


        }  // end x-tickmark loop


// Add name of x-axis:

        var xNameXOffsetThis = xNameXOffset - charToPx * "Depth (km)".length;

        txtPrint("<span style='font-size:small'>" + xAxisName + "</span>",
                xNameXOffsetThis, xNameYOffset, zeroInt, zeroInt, zeroInt, areaId);


        // console.log("XAxis: " + axisParams[0]);
        return axisParams;

    };

    var YAxis = function(plotRow, plotCol,
            numYTicks, minYDataIn, maxYDataIn, yAxisName,
            areaId) {

        var axisParams = new Array(6);

        // Variables to handle normalization and rounding:
        var numParts = new Array(2);
        var mantissa;

        var xOffset = 100 + plotCol * (xRange + 150);
        var yOffset = 100 + yRangeT + yOffsetT + plotRow * (yRange + 120);



        axisParams[0] = xOffset;
        axisParams[4] = yOffset;

        var xOffsetStr = numToPxStrng(xOffset);
        var yOffsetStr = numToPxStrng(yOffset);
        var xLowerYOffset = yOffset + yRange;
        var xLowerYOffsetStr = numToPxStrng(xLowerYOffset);


        //x-axis name properties:

        var yTickXOffset = xOffset - tickHeight / 2; //height and width reversed for y-ticks
        var yTickXOffsetStr = numToPxStrng(yTickXOffset);
        var yLabelXOffset = xOffset - 3 * labelHeight; //height & width reversed for y-ticks
        var yLabelXOffsetStr = numToPxStrng(yLabelXOffset);
        var yNameYOffset = yOffset + Math.floor(yRange / 2);
        //var yNameYOffsetStr = numToPxStrng(yNameYOffset);
        var yNameXOffset = xOffset - 90;
        //var yNameXOffsetStr = numToPxStrng(yNameXOffset);

        axisParams[5] = xLowerYOffset;



        // Create the LEFT y-axis element and set its style attributes:

        var yId = document.createElement("div");
        yId.class = "yaxis";
        yId.style.position = "absolute";
        yId.style.width = "1px";
        yId.style.height = yRangeStr;
        yId.style.marginLeft = xOffsetStr;
        yId.style.marginTop = yOffsetStr;
        yId.style.border = "1px black solid";
        //Draw the y-axis:
        //masterId.appendChild(yId);
        areaId.appendChild(yId);


        numParts = standForm(maxYDataIn);
        mantissa = rounder(numParts[0], 1, "up");
        maxYData = mantissa * Math.pow(10.0, numParts[1]);
        numParts = standForm(minYDataIn);
        mantissa = rounder(numParts[0], 1, "down");
        minYData = mantissa * Math.pow(10.0, numParts[1]);
        var rangeYData = maxYData - minYData;
        var deltaYData = rangeYData / (1.0 * numYTicks);
        numParts = standForm(deltaYData);
        mantissa = rounder(numParts[0], 1, "down");
        deltaYData = mantissa * Math.pow(10.0, numParts[1]);
        var deltaYPxl = yRange / numYTicks;


        axisParams[1] = rangeYData;
        axisParams[2] = deltaYData;
        axisParams[3] = deltaYPxl;

        var ii;
        for (var i = 0; i <= numYTicks; i++) {

            ii = 1.0 * i;
            var yTickPos = ii * deltaYPxl;
            // Doesn't work - ?? var yTickVal = minYDataRnd + (ii * deltaDataRnd);
            var yTickVal = minYData + (ii * deltaYData);
            var yTickRound = yTickVal.toPrecision(2);
            //var yTickRound = yTickVal;
            var yTickValStr = yTickRound.toString(10);
            // vertical position in pixels - data values increase upward:
            var yShift = xLowerYOffset - yTickPos;
//stringify and add unit:
            var yShiftStr = numToPxStrng(yShift);
// Make the y-tick mark, Teff:

            var yTickId = document.createElement("div");
            yTickId.class = "ytick";
            yTickId.style.position = "absolute";
            yTickId.style.display = "block";
            yTickId.style.marginLeft = yTickXOffsetStr;
            yTickId.style.border = tickBorder;
// Note that for y ticks, the height and width are reversed!:
            yTickId.style.width = tickHeightStr;
            yTickId.style.height = tickWidth;
            yTickId.style.zIndex = 0;
            yTickId.style.marginTop = yShiftStr;
//Append the tickmark to the axis element
            //masterId.appendChild(yTickId);
            areaId.appendChild(yTickId);
            //Make the y-tick label:

            txtPrint("<span style='font-size:small'>" + yTickValStr + "</span>",
                    yLabelXOffset, yShift, zeroInt, zeroInt, zeroInt, areaId);


        }  // end y-tickmark loop, j


// Add name of LEFT y-axis:

        txtPrint("<span style='font-size:small'>" + yAxisName + "</span>",
                yNameXOffset, yNameYOffset, zeroInt, zeroInt, zeroInt, areaId);


        //console.log("YAxis: " + axisParams[0]);
        return axisParams;

    };


    //   var testVal = -1.26832e7;
    //   var numParts = standForm(testVal);
    //  console.log("mantissa= " + numParts[0] + " exponent= " + numParts[1]);
//
    //   var roundVal = rounder(numParts[0], 1, "up");
    //   console.log("roundVal= " + roundVal);


// Plot one: depth vs Temp


    var plotRow = 1;
    var plotCol = 2;

    var numXTicks = 6;
    var minXData = (1.0E-5 * depths[0]) - 100.0; // cm to km
    var maxXData = (1.0E-5 * depths[numDeps - 1]) + 100.0; // cm to km


    var xAxisName = "Depth (km)";

    var numYTicks = 6;
    var minYData = temp[0][0] - 1000.0;
    var maxYData = temp[0][numDeps - 1] + 1000.0;


    var yAxisName = "<em>T</em><sub>Kin</sub> (K)";



    var xAxisParams = XAxis(plotRow, plotCol,
            numXTicks, minXData, maxXData, xAxisName,
            plotOneId);

    var yAxisParams = YAxis(plotRow, plotCol,
            numYTicks, minYData, maxYData, yAxisName,
            plotOneId);

    xOffset = xAxisParams[0];
    yOffset = yAxisParams[4];
    var rangeXData = xAxisParams[1];
    var deltaXData = xAxisParams[2];
    var deltaXPxl = xAxisParams[3];
    var rangeYData = yAxisParams[1];
    var deltaYData = yAxisParams[2];
    var deltaYPxl = yAxisParams[3];
    var xLowerYOffset = xAxisParams[5];


// Tau=1 cross-hair

    var tTau1 = tauPoint(numDeps, tauRos, 1.0);

    // Vertical bar:
    klmtrs = 1.0E-5 * depths[tTau1]; // cm to km


    xShift = YBar(plotRow, plotCol,
            klmtrs, minXData, maxXData,
            "#777777", plotOneId);
    //console.log("Bar: xShift = " + xShift);

    yShift = XBar(plotRow, plotCol,
            temp[0][tTau1], minYData, maxYData,
            "#777777", plotOneId);


    // Add label
    //txtPrint("<span title='Rosseland mean optical depth' style='font-size:small; color:#444444'>&#964<sub>Ros</sub>=1</span>", 
    //  xShift + 5, yShift - 25, zeroInt, zeroInt, zeroInt, masterId);
    txtPrint("<span title='Rosseland mean optical depth' style='font-size:small; color:#444444'>&#964<sub>Ros</sub>=1</span>",
            xShift + 5, yShift - 175, zeroInt, zeroInt, zeroInt, plotOneId);
// Tau_lambda(lambda_0) = 1 cross-hair

    // Get depth index where monochromatic line center Tau_l ~ 1:
    var indxLam0 = keyLambds[0]; // line center
    var tauLam0r0 = new Array(numDeps);
    var tauLam0r1 = new Array(numDeps);
    var tauLam0 = [tauLam0r0, tauLam0r1];
    for (var i = 0; i < numDeps - 1; i++) {
        tauLam0[0][i] = Math.exp(logTauL[indxLam0][i]);
        tauLam0[1][i] = logTauL[indxLam0][i];
    }
    var tTauL1 = tauPoint(numDeps, tauLam0, 1.0);


    // Vertical bar:
    klmtrs = 1.0E-5 * depths[tTauL1]; // cm to km


    xShift = YBar(plotRow, plotCol,
            klmtrs, minXData, maxXData,
            "#00FF77", plotOneId);
    //console.log("Bar: xShift = " + xShift);

    yShift = XBar(plotRow, plotCol,
            temp[0][tTauL1], minYData, maxYData,
            "#00FF77", plotOneId);
    //console.log("Bar: yShift = " + yShift);

    // Add label
    //txtPrint("<span style='font-size:small'>&#964<sub>&#955_0</sub>=1</span>", xShift - 35, yShift - 25, 0, 255, 100, masterId);
    txtPrint("<span style='font-size:small'>&#964<sub>&#955_0</sub>=1</span>", xShift - 35, yShift - 175, 0, 255, 100, plotOneId);
    var titleYPos = xLowerYOffset - 1.15 * yRange;
    var titleXPos = 1.02 * xOffset;
    //txtPrint("<span style='font-size:normal; color:blue'>Gas temperature <em>vs</em> depth </span>",
    //        titleXPos, titleYPos - 35, zeroInt, zeroInt, zeroInt, masterId);
    txtPrint("<span style='font-size:normal; color:blue'>Gas temperature <em>vs</em> depth </span>",
            titleXPos, titleYPos - 35, zeroInt, zeroInt, zeroInt, plotOneId);

    //Data loop - plot the result!

    var dSize = 5.0; //plot point size
    var opac = 1.0; //opacity
    // RGB color
    var r255 = 0;
    var g255 = 0;
    var b255 = 255; //blue 

    var ii;
    var klmtrs;
    for (var i = 0; i < numDeps; i++) {

        ii = 1.0 * i;
        klmtrs = 1.0E-5 * depths[i]; // cm to km
        var xTickPos = xRange * (klmtrs - minXData) / rangeXData; // pixels   

        // horizontal position in pixels - data values increase rightward:
        var xShift = xOffset + xTickPos;
////stringify and add unit:
//        var xShiftStr = numToPxStrng(xShift);

        var yTickPos = yRange * (temp[0][i] - minYData) / rangeYData;
        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
////stringify and add unit:
//       var yShiftStr = numToPxStrng(yShift);

        plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotOneId);
    }

    //masterId.appendChild(plotOneId);



// ****************************************




    // Plot two: log(Tau) vs Temp
    var plotRow = 0;
    var plotCol = 2;


    var numXTicks = 6;
    var minXData = logE * tauRos[1][0] - 0.5;
    var maxXData = logE * tauRos[1][numDeps - 1] + 0.5;


    var xAxisName = "<span title='Rosseland mean optical depth'><a href='http://en.wikipedia.org/wiki/Optical_depth_%28astrophysics%29' target='_blank'>Log<sub>10</sub> &#964<sub>Ros</sub></a></span>";

    var numYTicks = 6;
    var minYData = temp[0][0] - 1000.0;
    var maxYData = temp[0][numDeps - 1] + 1000.0;


    var yAxisName = "<em>T</em><sub>Kin</sub> (K)";

    //washer(xRange, xOffset, yRange, yOffset, wColor, plotTwoId);

    var xAxisParams = XAxis(plotRow, plotCol,
            numXTicks, minXData, maxXData, xAxisName,
            plotTwoId);

    var yAxisParams = YAxis(plotRow, plotCol,
            numYTicks, minYData, maxYData, yAxisName,
            plotTwoId);

    xOffset = xAxisParams[0];
    yOffset = xAxisParams[4];
    var rangeXData = xAxisParams[1];
    var deltaXData = xAxisParams[2];
    var deltaXPxl = xAxisParams[3];
    var rangeYData = yAxisParams[1];
    var deltaYData = yAxisParams[2];
    var deltaYPxl = yAxisParams[3];
    var xLowerYOffset = xAxisParams[5];


    // Tau=1 cross-hair

    var tTau1 = tauPoint(numDeps, tauRos, 1.0);


    xShift = YBar(plotRow, plotCol,
            logE * tauRos[1][tTau1], minXData, maxXData,
            "#777777", plotTwoId);
    //console.log("Bar: xShift = " + xShift);

    yShift = XBar(plotRow, plotCol,
            temp[0][tTau1], minYData, maxYData,
            "#777777", plotTwoId);

    // Add label
    //txtPrint("<span style='font-size:small; color:#444444'>&#964<sub>Ros</sub>=1</span>",
    //        xShift + 10, yShift - 25, zeroInt, zeroInt, zeroInt, masterId);
    txtPrint("<span style='font-size:small; color:#444444'>&#964<sub>Ros</sub>=1</span>",
            xShift + 10, yShift - 175, zeroInt, zeroInt, zeroInt, plotTwoId);
    // Tau_lambda(lambda_0) = 1 cross-hair

    // Get depth index where monochromatic line center Tau_l ~ 1:
    var indxLam0 = keyLambds[0]; // line center
    var tauLam0r0 = new Array(numDeps);
    var tauLam0r1 = new Array(numDeps);
    var tauLam0 = [tauLam0r0, tauLam0r1];
    for (var i = 0; i < numDeps - 1; i++) {
        tauLam0[0][i] = Math.exp(logTauL[indxLam0][i]);
        tauLam0[1][i] = logTauL[indxLam0][i];
    }
    var tTauL1 = tauPoint(numDeps, tauLam0, 1.0);



    xShift = YBar(plotRow, plotCol,
            logE * tauRos[1][tTauL1], minXData, maxXData,
            "#00FF77", plotTwoId);
    //console.log("Bar: xShift = " + xShift);

    yShift = XBar(plotRow, plotCol,
            temp[0][tTauL1], minYData, maxYData,
            "#00FF77", plotTwoId);

    // Add label
    //txtPrint("<span style='font-size:small'>&#964<sub>&#955_0</sub>=1</span>", 
    //   xShift - 35, yShift - 25, 0, 255, 100, masterId);
    txtPrint("<span style='font-size:small'>&#964<sub>&#955_0</sub>=1</span>",
            xShift - 35, yShift - 175, 0, 255, 100, plotTwoId);

    var titleYPos = xLowerYOffset - 1.15 * yRange;
    var titleXPos = 1.02 * xOffset;
    //txtPrint("<span style='font-size:normal; color:blue'>Gas temperature <em>vs</em> log &#964 </span>",
    //        titleXPos, titleYPos - 40, zeroInt, zeroInt, zeroInt, masterId);
    txtPrint("<span style='font-size:normal; color:blue'>Gas temperature <em>vs</em> log &#964 </span>",
            titleXPos, titleYPos - 40, zeroInt, zeroInt, zeroInt, plotTwoId);

    //Data loop - plot the result!

    var dSize = 5.0; //plot point size
    var opac = 1.0; //opacity
    // RGB color
    var r255 = 0;
    var g255 = 0;
    var b255 = 255; //blue

    var ii;
    for (var i = 0; i < numDeps; i++) {

        ii = 1.0 * i;
        var xTickPos = xRange * (logE * tauRos[1][i] - minXData) / rangeXData; // pixels   

        // horizontal position in pixels - data values increase rightward:
        var xShift = xOffset + xTickPos;
        ////stringify and add unit:
        //        var xShiftStr = numToPxStrng(xShift);

        var yTickPos = yRange * (temp[0][i] - minYData) / rangeYData;
        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
        ////stringify and add unit:
        //       var yShiftStr = numToPxStrng(yShift);

        plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotTwoId);


    }

    //  Loop over limb darkening sub-disks - largest to smallest, and add color-coded Tau(theta) = 1 markers

    var iLamMinMax = minMax2(flux);
    var iLamMax = iLamMinMax[1];
    dSize = 8.0;
    for (var i = numThetas - 1; i >= 0; i--) {

        //  for (var i = numThetas2 - 1; i >= 0; i--) {

        ii = 1.0 * i;
        //     iCosThetaI = limbTheta1 - ii * limbDelta;
        //     iIntMaxI = interpol(iCosTheta, iIntMax, iCosThetaI);

        //numPrint(i, 50, 100 + i * 20, zeroInt, zeroInt, zeroInt, masterId);
        // LTE Eddington-Barbier limb darkening: I(Tau=0, cos(theta)=t) = B(T(Tau=t))
        var cosFctr = cosTheta[1][i];
        //  var cosFctr = iCosThetaI;
        //numPrint(cosFctr, 100, 100+i*20, zeroInt, zeroInt, zeroInt, masterId);
        var dpthIndx = tauPoint(numDeps, tauRos, cosFctr);
        //numPrint(dpthIndx, 100, 100+i*20, zeroInt, zeroInt, zeroInt, masterId);
        var tempDisk = temp[0][dpthIndx];
        //numPrint(tempDisk, 100, 100+i*20, zeroInt, zeroInt, zeroInt, masterId);
        var tempDisk2 = Math.max(tempDisk, minTeff);
        //numPrint(tempDisk2, 150, 100 + i * 20, zeroInt, zeroInt, zeroInt, masterId);
        //       var logLamPeak = Math.log(wien / tempDisk2); // lambda in cm

        // limb darkening intensity factor:
        var dark = (intens[iLamMax][i] / intens[iLamMax][0]);
        //var dark = iIntMaxI;

        //var starRGBHex = colorTemp(tempDisk2, minTeff, maxTeff, dark);
        var RGBArr = colorTemp(tempDisk2, minTeff, maxTeff, dark);
        var r255 = RGBArr[0];
        var g255 = RGBArr[1];
        var b255 = RGBArr[2];

        var xTickPos = xRange * (logE * tauRos[1][dpthIndx] - minXData) / rangeXData; // pixels   

        // horizontal position in pixels - data values increase rightward:
        var xShift = xOffset + xTickPos;
        ////stringify and add unit:
        //        var xShiftStr = numToPxStrng(xShift);

        var yTickPos = yRange * (temp[0][dpthIndx] - minYData) / rangeYData;
        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
        ////stringify and add unit:
        //       var yShiftStr = numToPxStrng(yShift);

        plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotTwoId);


    }

    // legend using dot of last color in loop directly above:
    var titleYPos = xLowerYOffset - 1.25 * yRange;
    var titleXPos = 1.02 * xOffset;
    plotPnt(titleXPos, titleYPos + 10, r255, g255, b255, opac, dSize, plotTwoId);
    plotPnt(titleXPos + 10, titleYPos + 10, 0.1 * r255, 0.1 * g255, 0.1 * b255, opac, dSize, plotTwoId);
    //txtPrint("<span title='Limb darkening depths of &#964_Ros(&#952) = 1'>&#964<sub>Ros</sub>(0 < &#952 < 90<sup>o</sup>) = 1</span>",
    //        titleXPos + 20, titleYPos - 5, zeroInt, zeroInt, zeroInt, masterId);
    txtPrint("<span title='Limb darkening depths of &#964_Ros(&#952) = 1'>&#964<sub>Ros</sub>(0 < &#952 < 90<sup>o</sup>) = 1</span>",
            titleXPos + 20, titleYPos - 5, zeroInt, zeroInt, zeroInt, plotTwoId);




    // ****************************************


    // Plot three: log(Tau) vs Pressure
    var plotRow = 1;
    var plotCol = 0;


    var numXTicks = 6;
    var minXData = logE * tauRos[1][0] - 0.5;
    var maxXData = logE * tauRos[1][numDeps - 1] + 0.5;


    var xAxisName = "<span title='Rosseland mean optical depth'><a href='http://en.wikipedia.org/wiki/Optical_depth_%28astrophysics%29' target='_blank'>Log<sub>10</sub> &#964<sub>Ros</sub></a></span>";


    // From Hydrostat.hydrostat:
    //press is a 4 x numDeps array:
    // rows 0 & 1 are linear and log *gas* pressure, respectively
    // rows 2 & 3 are linear and log *radiation* pressure
    // Don't use upper boundary condition as lower y-limit - use a couple of points below surface:
    var numYTicks = 6;
    // Build total P from P_Gas & P_Rad:
    var logPTot = new Array(numDeps);
    for (var i = 0; i < numDeps; i++) {
        logPTot[i] = Math.log(press[0][i] + press[2][i]);
        //console.log(logPTot[i]);
    }
    var minYData = logE * logPTot[1] - 2.5; // Avoid upper boundary condition [i]=0
    var maxYData = logE * logPTot[numDeps - 1] + 1.0;

    var yAxisName = "Log<sub>10</sub> <em>P</em> <br />(dynes <br />cm<sup>-2</sup>)";

    //washer(xRange, xOffset, yRange, yOffset, wColor, plotThreeId);

    var xAxisParams = XAxis(plotRow, plotCol,
            numXTicks, minXData, maxXData, xAxisName,
            plotThreeId);

    var yAxisParams = YAxis(plotRow, plotCol,
            numYTicks, minYData, maxYData, yAxisName,
            plotThreeId);

    xOffset = xAxisParams[0];
    yOffset = xAxisParams[4];
    var rangeXData = xAxisParams[1];
    var deltaXData = xAxisParams[2];
    var deltaXPxl = xAxisParams[3];
    var rangeYData = yAxisParams[1];
    var deltaYData = yAxisParams[2];
    var deltaYPxl = yAxisParams[3];
    var xLowerYOffset = xAxisParams[5];


    var legendYPos = xLowerYOffset - 1.05 * yRange;
    var legendXPos = 1.1 * xOffset;

    txtPrint("log Pressure <em>vs</em> log &#964: <span style='color:blue' title='Total pressure'><strong><em>P</em><sub>Tot</sub></strong></span> "
            + " <a href='http://en.wikipedia.org/wiki/Gas_laws' target='_blank'><span style='color:#00FF88' title='Gas pressure'><em>P</em><sub>Gas</sub></span></a> "
            + " <a href='http://en.wikipedia.org/wiki/Radiation_pressure' target='_blank'><span style='color:red' title='Radiation pressure'><em>P</em><sub>Rad</sub></span></a>",
            legendXPos - 20, legendYPos - 20, zeroInt, zeroInt, zeroInt, plotThreeId);

    //txtPrint(" <a href='http://en.wikipedia.org/wiki/Gas_laws' target='_blank'><span title='Gas pressure'><em>P</em><sub>Gas</sub></span></a> ",
    //        legendXPos + 140, legendYPos - 20, 0, 255, 0, masterId);

    //Data loop - plot the result!

    var dSize = 6.0; //plot point size
    var dSizeG = 4.0;
    var opac = 1.0; //opacity
    // RGB color
    // PTot:
    var r255 = 0;
    var g255 = 0;
    var b255 = 255; //blue 
    // PGas:
    var r255G = 0;
    var g255G = 255;
    var b255G = 100; //green
    // PRad:
    var r255R = 255;
    var g255R = 0;
    var b255R = 0; //red

    var ii;
    // Avoid upper boundary at i=0
    for (var i = 1; i < numDeps; i++) {

        ii = 1.0 * i;
        var xTickPos = xRange * (logE * tauRos[1][i] - minXData) / rangeXData; // pixels   

        // horizontal position in pixels - data values increase rightward:
        var xShift = xOffset + xTickPos;
        ////stringify and add unit:
        //        var xShiftStr = numToPxStrng(xShift);

        var yTickPos = yRange * (logE * logPTot[i] - minYData) / rangeYData;
        var yTickPosG = yRange * (logE * press[1][i] - minYData) / rangeYData;
        var yTickPosR = yRange * (logE * press[3][i] - minYData) / rangeYData;
        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
        var yShiftG = xLowerYOffset - yTickPosG;
        var yShiftR = xLowerYOffset - yTickPosR;
        ////stringify and add unit:
        //       var yShiftStr = numToPxStrng(yShift);

        plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotThreeId);
        plotPnt(xShift, yShiftG, r255G, g255G, b255G, opac, dSizeG, plotThreeId);
        plotPnt(xShift, yShiftR, r255R, g255R, b255R, opac, dSizeG, plotThreeId);
    }

    // Tau=1 cross-hair

    var tTau1 = tauPoint(numDeps, tauRos, 1.0);

    xShift = YBar(plotRow, plotCol,
            logE * tauRos[1][tTau1], minXData, maxXData,
            "#777777", plotThreeId);
    //console.log("Bar: xShift = " + xShift);

    yShift = XBar(plotRow, plotCol,
            logE * logPTot[tTau1], minYData, maxYData,
            "#777777", plotThreeId);

    txtPrint("<span style='font-size:small; color:#444444'>&#964<sub>Ros</sub>=1</span>",
            xShift + 10, yShift - 50, zeroInt, zeroInt, zeroInt, plotThreeId);

    // ****************************************


    // Plot four: Limb darkening

    var plotRow = 1;
    var plotCol = 1;


    var numXTicks = 9;
    var minXData = 180.0 * Math.acos(cosTheta[1][0]) / Math.PI;
    var maxXData = 180.0 * Math.acos(cosTheta[1][numThetas - 1]) / Math.PI;

    var xAxisName = "&#952 (<sup>o</sup>)";

    var numYTicks = 4;
    var minYData = 0.0;
    var maxYData = 1.0;

    var yAxisName = "<span title='Monochromatic surface specific intensity'><a href='http://en.wikipedia.org/wiki/Specific_radiative_intensity' target='_blank'><em>I</em><sub>&#955</sub>(&#952)/<br /><em>I</em><sub>&#955</sub>(0)</a></span>";

    //washer(xRange, xOffset, yRange, yOffset, wColor, plotFourId);

    var xAxisParams = XAxis(plotRow, plotCol,
            numXTicks, minXData, maxXData, xAxisName,
            plotFourId);

    var yAxisParams = YAxis(plotRow, plotCol,
            numYTicks, minYData, maxYData, yAxisName,
            plotFourId);

    xOffset = xAxisParams[0];
    yOffset = xAxisParams[4];
    var rangeXData = xAxisParams[1];
    var deltaXData = xAxisParams[2];
    var deltaXPxl = xAxisParams[3];
    var rangeYData = yAxisParams[1];
    var deltaYData = yAxisParams[2];
    var deltaYPxl = yAxisParams[3];
    var xLowerYOffset = xAxisParams[5];


    // Add legend annotation:

    var legendYPos = xLowerYOffset - 0.35 * yRange;
    var legendXPos = 1.02 * xOffset;

    var iLamMinMax = minMax2(flux);
    var iLamMax = iLamMinMax[1];
    var lamMax = lambdaScale[iLamMax].toPrecision(3);
    var lamMaxStr = lamMax.toString(10);
    var lam1 = lambdaScale[0].toPrecision(3);
    var lam1Str = lam1.toString(10);
    var lamN = lambdaScale[numLams - 1].toPrecision(3);
    var lamNStr = lamN.toString(10);
    var lam0r = lam0.toPrecision(3);
    var lam0rStr = lam0r.toString(10);

    txtPrint("<span style='font-size:small'><span style='color:#00FF88'>&#955<sub>Max</sub> = " + lamMaxStr + "nm</span><br /> "
            + " <span style='color:blue'>&#955 = " + lam1Str + "nm</span><br /> "
            + " <span style='color:red'>&#955 = " + lamNStr + "nm</span><br />"
            + " <span style='color:#444444'>line &#955<sub>0</sub> = " + lam0rStr + "nm</span></span>",
            legendXPos, legendYPos, zeroInt, zeroInt, zeroInt, plotFourId);

    // Add title annotation:


    var titleYPos = xLowerYOffset - 1.15 * yRange;
    var titleXPos = 1.02 * xOffset;

    txtPrint("<span style='font-size:normal; color:blue'><a href='http://en.wikipedia.org/wiki/Limb_darkening' target='_blank'>Limb darkening and reddening</a></span>",
            titleXPos, titleYPos, zeroInt, zeroInt, zeroInt, plotFourId);

    //Data loop - plot the result!

    var dSize = 8.0; //plot point size
    var dSize0 = 6.0;
    var opac = 1.0; //opacity
    // RGB color
    // PTot:
    var r255 = 0;
    var g255 = 255;
    var b255 = 100; //green 
    // PGas:
    var r2550 = 0;
    var g2550 = 0;
    var b2550 = 255; //blue
    // PRad:
    var r255N = 255;
    var g255N = 0;
    var b255N = 0; //red

    // // Interpolate line profile onto finer grid for smooth look:  (JavaScript plotting won't 'connect-the-dots' )  
    // var limbTheta1 = cosTheta[1][0];  //1.0
    // var limbTheta2 = cosTheta[1][numThetas - 1]; //>~ 0
    // var limbRange = (limbTheta1 - limbTheta2);
    // var limbDelta = 0.05; //cosTheta
    // var numThetas2 = limbRange / limbDelta + 1;
    //Data loop - plot the result!

    // Interpolation variables:
    //  var iInt0 = new Array(numThetas);
    // var iIntN = new Array(numThetas);
    //  var iIntMax = new Array(numThetas);
    // var iIntLam0 = new Array(numThetas);
    //  var iCosTheta = new Array(numThetas);
    //  var ii;
    //  for (var i = 0; i < numThetas; i++) {
    //
    //      iInt0[i] = intens[0][i] / intens[0][0];
    //      iIntN[i] = intens[numLams - 1][i] / intens[numLams - 1][0];
    //     iIntMax[i] = intens[iLamMax][i] / intens[iLamMax][0];
    //     iIntLam0[i] = lineIntens[keyLambds[0]][i] / lineIntens[keyLambds[0]][0];
    //     iCosTheta[i] = cosTheta[1][i];
    //  }

    // var iCosThetaI;
    // var iIntMaxI, iInt0I, iIntNI, iIntLam0I;


    var ii;
    //for (var i = 0; i < numThetas2; i++) {
    for (var i = 0; i < numThetas; i++) {

        ii = 1.0 * i;

        //     iCosThetaI = limbTheta1 - ii * limbDelta;
        //    iInt0I = interpol(iCosTheta, iInt0, iCosThetaI);
        //    iIntNI = interpol(iCosTheta, iIntN, iCosThetaI);
        //     iIntMaxI = interpol(iCosTheta, iIntMax, iCosThetaI);
        //     iIntLam0I = interpol(iCosTheta, iIntLam0, iCosThetaI);

        var xTickPos = xRange * (180.0 * Math.acos(cosTheta[1][i]) / Math.PI - minXData) / rangeXData; // pixels   
        //var xTickPos = xRange * (180.0 * Math.acos(iCosThetaI) / Math.PI - minXData) / rangeXData; // pixels   
        // horizontal position in pixels - data values increase rightward:
        var xShift = xOffset + xTickPos;
        ////stringify and add unit:
        //        var xShiftStr = numToPxStrng(xShift);

        var yTickPos = yRange * ((intens[iLamMax][i] / intens[iLamMax][0]) - minYData) / rangeYData;
        var yTickPos0 = yRange * ((intens[0][i] / intens[0][0]) - minYData) / rangeYData;
        var yTickPosN = yRange * ((intens[numLams - 1][i] / intens[numLams - 1][0]) - minYData) / rangeYData;
        var yTickPosLn = yRange * (lineIntens[keyLambds[0]][i] / lineIntens[keyLambds[0]][0]);
        // var yTickPos = yRange * (iIntMaxI - minYData) / rangeYData;
        //  var yTickPos0 = yRange * (iInt0I - minYData) / rangeYData;
        //  var yTickPosN = yRange * (iIntNI - minYData) / rangeYData;
        // var yTickPosLn = yRange * (iIntLam0I);

        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
        var yShift0 = xLowerYOffset - yTickPos0;
        var yShiftN = xLowerYOffset - yTickPosN;
        var yShiftLn = xLowerYOffset - yTickPosLn;
        ////stringify and add unit:
        //       var yShiftStr = numToPxStrng(yShift);

        plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotFourId);
        plotPnt(xShift, yShift0, r2550, g2550, b2550, opac, dSize0, plotFourId);
        plotPnt(xShift, yShiftN, r255N, g255N, b255N, opac, dSize0, plotFourId);
        plotPnt(xShift, yShiftLn, 100, 100, 100, opac, dSize0, plotFourId); //gray

    }



    // Plot five: SED

    var plotRow = 2;
    var plotCol = 0;

    var numXTicks = 5;
    var minXData = lambdaScale[0] - 50.;
    var maxXData = lambdaScale[numLams - 1] + 100.;

    var xAxisName = "&#955 (nm)";

    var numYTicks = 4;
    //var minYData = intens[0][0];  // 
    //var maxYData = intens[numLams - 1][0];
    var minYData = 0.0;
    var maxYData = flux[0][iLamMax];

    var yAxisName = "<span title='Monochromatic surface flux'><a href='http://en.wikipedia.org/wiki/Spectral_flux_density' target='_blank'><em>F</em><sub>&#955</sub> <br /> ergs s<sup>-1</sup> cm<sup>-3</sup></a></span>";

    //(xRange, xOffset, yRange, yOffset, wColor, plotFiveId);

    var xAxisParams = XAxis(plotRow, plotCol,
            numXTicks, minXData, maxXData, xAxisName,
            plotFiveId);

    var yAxisParams = YAxis(plotRow, plotCol,
            numYTicks, minYData, maxYData, yAxisName,
            plotFiveId);

    xOffset = xAxisParams[0];
    yOffset = xAxisParams[4];
    var rangeXData = xAxisParams[1];
    var deltaXData = xAxisParams[2];
    var deltaXPxl = xAxisParams[3];
    var rangeYData = yAxisParams[1];
    var deltaYData = yAxisParams[2];
    var deltaYPxl = yAxisParams[3];
    var xLowerYOffset = xAxisParams[5];


    // Add legend annotation:


    var legendYPos = xLowerYOffset - 1.2 * yRange;
    var legendXPos = 1.05 * xOffset;

    var thet0 = 180.0 * Math.acos(cosTheta[1][0]) / Math.PI;
    var thet0lbl = thet0.toPrecision(2);
    var thet0Str = thet0lbl.toString();
    var thetN = 180.0 * Math.acos(cosTheta[1][numThetas - 2]) / Math.PI;
    var thetNlbl = thetN.toPrecision(2);
    var thetNStr = thetNlbl.toString();


    txtPrint("<span style='font-size:small'>"
            + "<span><em>F</em><sub>&#955</sub> (&#955<sub>Max</sub> = " + lamMaxStr + " nm)</span>, "
            + " <span><em>I</em><sub>&#955</sub>,</span> <span style='color:#444444'> &#952 = " + thet0Str + "<sup>o</sup></span>,  "
            + " <span style='color:#444444'>&#952 = " + thetNStr + "<sup>o</sup></span></span>",
            legendXPos, legendYPos + 10, zeroInt, zeroInt, zeroInt, plotFiveId);

    var titleYPos = xLowerYOffset - 1.15 * yRange;
    var titleXPos = 1.02 * xOffset;
    txtPrint("<span style='font-size:normal; color:blue'><a href='http://en.wikipedia.org/wiki/Spectral_energy_distribution' target='_blank'>\n\
Spectral energy distribution (SED)</a></span>",
            titleXPos, titleYPos - 25, zeroInt, zeroInt, zeroInt, plotFiveId);

    //Data loop - plot the result!

    var dSize = 6.0; //plot point size
    var dSize0 = 4.0;
    var opac = 1.0; //opacity
    // RGB color
    // PTot:
    var r255 = 0;
    var g255 = 0;
    var b255 = 0; //black
    // PGas:
    var r2550 = 90;
    var g2550 = 90;
    var b2550 = 90; //dark gray
    // PRad:
    var r255N = 120;
    var g255N = 120;
    var b255N = 120; //light gray

    var ii;
    // Avoid upper boundary at i=0

    for (var i = 0; i < numLams; i++) {

        ii = 1.0 * i;
        var xTickPos = xRange * (lambdaScale[i] - minXData) / rangeXData; // pixels   

        // horizontal position in pixels - data values increase rightward:
        var xShift = xOffset + xTickPos;
        ////stringify and add unit:
        //        var xShiftStr = numToPxStrng(xShift);

        var yTickPos = yRange * (flux[0][i] - minYData) / rangeYData;
        var yTickPos0 = yRange * (intens[i][0] - minYData) / rangeYData;
        var yTickPosN = yRange * (intens[i][numThetas - 2] - minYData) / rangeYData;
        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
        var yShift0 = xLowerYOffset - yTickPos0;
        var yShiftN = xLowerYOffset - yTickPosN;
        ////stringify and add unit:
        //       var yShiftStr = numToPxStrng(yShift);

        plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotFiveId);
        plotPnt(xShift, yShift0, r2550, g2550, b2550, opac, dSize0, plotFiveId);
        plotPnt(xShift, yShiftN, r255N, g255N, b255N, opac, dSize0, plotFiveId);
    }




    // Photometric bands centers


    var opac = 0.75;
    var opacStr = numToPxStrng(opac);
    var yTickPos = 0;
    var yShift = (xLowerYOffset - yRange) + yTickPos;
    var yShiftStr = numToPxStrng(yShift);
    var vBarWidth = 2; //pixels 
    var vBarHeight = yRange;
    var vBarWidthStr = numToPxStrng(vBarWidth);
    var vBarHeightStr = numToPxStrng(vBarHeight);

    var UBVRIBands = function(r255, g255, b255, band0) {

        var RGBHex = colHex(r255, g255, b255);
        // Vertical bar:
        var xTickPos = xRange * (band0 - minXData) / rangeXData; // pixels    
        var xShift = xOffset + xTickPos;
        var xShiftStr = numToPxStrng(xShift);
        var vCrossId = document.createElement("div");
        vCrossId.class = "cross";
        vCrossId.style.position = "absolute";
        vCrossId.style.display = "block";
        vCrossId.style.height = vBarHeightStr;
        vCrossId.style.width = vBarWidthStr;
        //hCrossId.style.borderRadius = "100%";
        vCrossId.style.opacity = opacStr;
        vCrossId.style.backgroundColor = RGBHex;
        vCrossId.style.marginLeft = xShiftStr;
        vCrossId.style.marginTop = yShiftStr;
        //Append the dot to the plot
        plotFiveId.appendChild(vCrossId);



    };

    //U:
    var r255 = 155;
    var g255 = 0;
    var b255 = 155; // violet

    UBVRIBands(r255, g255, b255, lamUBVRI[0][0]);

    //B:
    var r255 = 0;
    var g255 = 0;
    var b255 = 255; // blue
    UBVRIBands(r255, g255, b255, lamUBVRI[0][1]);

    //V:
    var r255 = 0;
    var g255 = 255;
    var b255 = 100; // green
    UBVRIBands(r255, g255, b255, lamUBVRI[0][2]);

    //R:
    var r255 = 255;
    var g255 = 0;
    var b255 = 0; // red
    UBVRIBands(r255, g255, b255, lamUBVRI[0][3]);

    //I:
    var r255 = 255;
    var g255 = 40;
    var b255 = 40; // dark red / brown ??
    UBVRIBands(r255, g255, b255, lamUBVRI[0][4]);

    // *********************************************

    // Plot six: Line profile

    var plotRow = 2;
    var plotCol = 1;



    var numXTicks = 6;
    var minXData = lineLambdas[0] - lam0;
    // CAUTION; numPoints-1st value holds the line centre monochromatic *continuum* flux for normalization:
    var maxXData = lineLambdas[numPoints - 2] - lam0;


    var xAxisName = "&#916 &#955 (nm)";

    var numYTicks = 6;
    //var minYData = intens[0][0];  // 
    //var maxYData = intens[numLams - 1][0];
    var minYData = 0.0;
    var maxYData = 1.2;

    var yAxisName = "<span title='Continuum normalized flux'><a href='http://en.wikipedia.org/wiki/Spectral_flux_density' target='_blank'><em>F</em><sub>&#955</sub>/<br /><em>F</em><sup>C</sup><sub>&#955 0</sub></a></span>";

    //washer(xRange, xOffset, yRange, yOffset, wColor, plotSixId);

    var xAxisParams = XAxis(plotRow, plotCol,
            numXTicks, minXData, maxXData, xAxisName,
            plotSixId);

    var yAxisParams = YAxis(plotRow, plotCol,
            numYTicks, minYData, maxYData, yAxisName,
            plotSixId);

    xOffset = xAxisParams[0];
    yOffset = xAxisParams[4];
    var rangeXData = xAxisParams[1];
    var deltaXData = xAxisParams[2];
    var deltaXPxl = xAxisParams[3];
    var rangeYData = yAxisParams[1];
    var deltaYData = yAxisParams[2];
    var deltaYPxl = yAxisParams[3];
    var xLowerYOffset = xAxisParams[5];


    // Add legend annotation:


    var legendYPos = xLowerYOffset - 1.2 * yRange;
    var legendXPos = 1.05 * xOffset;

    var lam0lbl = lam0.toPrecision(2);
    var lam0Str = lam0lbl.toString();
    var thetN = 180.0 * Math.acos(cosTheta[1][5]) / Math.PI;
    var thetNlbl = thetN.toPrecision(5);
    var thetNStr = thetNlbl.toString();

    txtPrint("<span style='font-size:small'><span><em>F</em><sub>&#955</sub>, &#955<sub>0</sub> = " + lam0Str + " nm</span><br /> "
            + " <span><em>I</em><sub>&#955</sub>,</span> <span style='color:green'> &#955 = " + thet0Str + "<sup>o</sup></span>,  "
            + " <span style='color:red'>&#952 = " + thetNStr + "<sup>o</sup></span></span>",
            legendXPos, legendYPos, zeroInt, zeroInt, zeroInt, plotSixId);


    var dSize = 5.0; //plot point size
    var dSize0 = 3.0;
    var opac = 1.0; //opacity
    // RGB color
    // PTot:
    var r255 = 0;
    var g255 = 0;
    var b255 = 0; //black
    // PGas:
    var r2550 = 0;
    var g2550 = 255;
    var b2550 = 100; // green
    // PRad:
    var r255N = 255;
    var g255N = 0;
    var b255N = 0; //red



    // Continuum:

    var opacc = 0.5;
    var opaccStr = numToPxStrng(opac);
    var r255c = 130;
    var g255c = 130;
    var b255c = 130; // gray
    var RGBHexc = colHex(r255c, g255c, b255c);

    // Horizontal bar:

    var one = 1.0;
    yShift = XBar(plotRow, plotCol,
            one, minYData, maxYData,
            RGBHexc, plotSixId);

    //   
    //    // Line center, lambda_0 & HWHM lambda
    //    
    //    
    //    var opac = 0.75;
    //    var opacStr = numToPxStrng(opac);
    //    
    //    var yTickPos = 0;
    //    var yShift = (xLowerYOffset - yRange) + yTickPos;
    //    var yShiftStr = numToPxStrng(yShift);
    //     
    //     var vBarWidth = 2; //pixels 
    //     var vBarHeight = yRange;
    //     var vBarWidthStr = numToPxStrng(vBarWidth);
    //    var vBarHeightStr = numToPxStrng(vBarHeight);
    //     
    //    //lambda_0 line center:
    //     var r255 = 100;
    //     var g255 = 100;
    //    var b255 = 100; // gray
    //     var RGBHex = colHex(r255, g255, b255);
    //     
    //     // Vertical bar: line center, lambda_0
    //     //var xTickPos = xRange * (lineLambdas[numPoints/2] - lam0 - minXData) / rangeXData;  // pixels    
    //     var xTickPos = xRange * (lineLambdas[keyLambds[0]] - lam0 - minXData) / rangeXData;  // pixels 
    //     //var xTickPos = xRange * (0.0 - minXData) / rangeXData;  // pixels 
    //    var xShift = xOffset + xTickPos;
    //    var xShiftStr = numToPxStrng(xShift);
    //     
    //    
    //   
    //    // Vertical bar: half-power width
    //    var xTickPos = xRange * (lineLambdas[keyLambds[1]] - lam0 - minXData) / rangeXData;  // pixels 
    ///     var xShift = xOffset + xTickPos;
    //    var xShiftStr = numToPxStrng(xShift);
    //    

    //    

    var titleYPos = xLowerYOffset - 1.15 * yRange;
    var titleXPos = 1.02 * xOffset;
    txtPrint("<span style='font-size:normal; color:blue'><a href='http://en.wikipedia.org/wiki/Spectral_line' target='_blank'>Spectral line profile </a></span>",
            titleXPos, titleYPos - 35, zeroInt, zeroInt, zeroInt, plotSixId);

    // Data loop below in here instead

    // Interpolate line profile onto finer grid for smooth look:  (JavaScript plotting won't 'connect-the-dots' )  
    var lineLam1 = lineLambdas[0];
    // CAUTION; numPoints-1st value holds the line centre monochromatic *continuum* flux for normalization:
    var lineLam2 = lineLambdas[numPoints - 2];
    var lineRange = (lineLam2 - lineLam1);
    var lineDelta = 0.001; //nm
    var numPoints2 = lineRange / lineDelta;
    //Data loop - plot the result!

    // Interpolation variables:
    // CAUTION; numPoints-1st value holds the line centre monochromatic *continuum* flux for normalization:
    var lnFlx = new Array(numPoints - 1);
    var lnInt0 = new Array(numPoints - 1);
    var lnIntN = new Array(numPoints - 1);
    var lnLam = new Array(numPoints - 1);
    var ii;
    for (var i = 0; i < numPoints - 1; i++) {
        lnFlx[i] = lineFlux[0][i] / lineFlux[0][numPoints - 1];
        lnInt0[i] = lineIntens[i][5] / lineIntens[numPoints - 1][5];
        lnIntN[i] = lineIntens[i][numThetas - 2] / lineIntens[numPoints - 1][numThetas - 2];
        lnLam[i] = lineLambdas[i];
    }

    var lnLamI;
    var lnFlxI, lnInt0I, lnIntNI;

    // CAUTION; numPoints-1st value holds the line centre monochromatic *continuum* flux for normalization:
    //for (var i = 0; i < numPoints - 1; i++) {
    for (var i = 0; i < numPoints2 - 1; i++) {

        ii = 1.0 * i;
        lnLamI = ii * lineDelta + lineLam1;
        lnFlxI = interpol(lnLam, lnFlx, lnLamI);
        lnInt0I = interpol(lnLam, lnInt0, lnLamI);
        lnIntNI = interpol(lnLam, lnIntN, lnLamI);
        //var xTickPos = xRange * (lineLambdas[i] - lam0 - minXData) / rangeXData;  // pixels   
        var xTickPos = xRange * (lnLamI - lam0 - minXData) / rangeXData;
        // horizontal position in pixels - data values increase rightward:
        var xShift = xOffset + xTickPos;
        ////stringify and add unit:
        //        var xShiftStr = numToPxStrng(xShift);



        var yTickPos = yRange * (lnFlxI - minYData) / rangeYData;
        var yTickPos0 = yRange * (lnInt0I - minYData) / rangeYData;
        var yTickPosN = yRange * (lnIntNI - minYData) / rangeYData;
        //var yTickPos = yRange * (lineFlux[0][i] / lineFlux[0][numPoints - 1] - minYData) / rangeYData;
        //var yTickPos0 = yRange * (lineIntens[i][5] / lineIntens[numPoints - 1][5] - minYData) / rangeYData;
        //var yTickPosN = yRange * (lineIntens[i][numThetas - 2] / lineIntens[numPoints - 1][numThetas - 2] - minYData) / rangeYData;
        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
        var yShift0 = xLowerYOffset - yTickPos0;
        var yShiftN = xLowerYOffset - yTickPosN;
        ////stringify and add unit:
        //       var yShiftStr = numToPxStrng(yShift);

        plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotSixId);
        plotPnt(xShift, yShift0, r2550, g2550, b2550, opac, dSize0, plotSixId);
        plotPnt(xShift, yShiftN, r255N, g255N, b255N, opac, dSize0, plotSixId);
    }




    //***************************



    // Plot seven - image of limb-darkened and limb-colored stellar disk


    var plotRow = 0;
    var plotCol = 1;


    var xOffset = 100 + plotCol * (xRange + 150) + xRange / 2;
    var yOffset = 100 + yRangeT + yOffsetT + plotRow * (yRange + 120);
    var yOffsetStr = numToPxStrng(yOffset);
    var xLowerYOffset = yOffset + yRange / 2;
    var xTickYOffset = xLowerYOffset - tickHeight / 2;
    var xTickYOffsetStr = numToPxStrng(xTickYOffset);
    var xLabelYOffset = xLowerYOffset + labelHeight;
    var xLabelYOffsetStr = numToPxStrng(xLabelYOffset);
    //x-axis name properties:
    var xNameYOffset = xLowerYOffset + 2 * labelHeight;
    var xNameYOffsetStr = numToPxStrng(xNameYOffset);
    var xNameXOffset = Math.floor(xRange / 2) + xOffset;
    var xNameXOffsetStr = numToPxStrng(xNameXOffset);
    var yTickXOffset = xOffset - tickHeight / 2; //height and width reversed for y-ticks
    var yTickXOffsetStr = numToPxStrng(yTickXOffset);
    var yLabelXOffset = xOffset - 3 * labelHeight; //height & width reversed for y-ticks
    var yLabelXOffsetStr = numToPxStrng(yLabelXOffset);
    var yNameYOffset = yOffset + Math.floor(yRange / 2);
    var yNameYOffsetStr = numToPxStrng(yNameYOffset);
    var yNameXOffset = xOffset - 120;
    var yNameXOffsetStr = numToPxStrng(yNameXOffset);
    // Convert solar radii to pixels:

    var radiusScale = 20; //solar_radii-to-pixels!
    var logScale = 100; //amplification factor for log pixels


    // 
    // Star radius in pixels:

    //    var radiusPx = (radiusScale * radius);  //linear radius
    var radiusPx = logScale * logTen(radiusScale * radius); //logarithmic radius
    var radiusPx = Math.ceil(radiusPx);

    var titleYPos = xLowerYOffset - yRange + 40;
    //var titleXPos = xOffset - xOffset / 2;
    var titleXPos = xOffset;
    var thet1, thet2;
    var thet3;

    washer(xOffset - xRange / 2, yOffset, wColor, plotSevenId);

    // Add title annotation:

    //var titleYPos = xLowerYOffset - 1.15 * yRange;
    //var titleXPos = 1.02 * xOffset;

    txtPrint("<span style='font-size:normal; color:blue'><a href='http://en.wikipedia.org/wiki/Limb_darkening' target='_blank'>Limb darkened and reddened stellar disk <br />\n\
     Logarithmic radius </a></span>",
            titleXPos - 100, titleYPos - 15, zeroInt, zeroInt, zeroInt, plotSevenId);

    txtPrint("<span style='font-size:normal; color:black'>&#952 = </span>",
            titleXPos + 30, titleYPos + 5, zeroInt, zeroInt, zeroInt, plotSevenId);

    //  Loop over limb darkening sub-disks - largest to smallest
    for (var i = numThetas - 1; i >= 1; i--) {

        //  for (var i = numThetas2 - 1; i >= 0; i--) {

        ii = 1.0 * i;
        //     iCosThetaI = limbTheta1 - ii * limbDelta;
        //     iIntMaxI = interpol(iCosTheta, iIntMax, iCosThetaI);

        //numPrint(i, 50, 100 + i * 20, zeroInt, zeroInt, zeroInt, masterId);
        // LTE Eddington-Barbier limb darkening: I(Tau=0, cos(theta)=t) = B(T(Tau=t))
        var cosFctr = cosTheta[1][i];
        //  var cosFctr = iCosThetaI;
        //numPrint(cosFctr, 100, 100+i*20, zeroInt, zeroInt, zeroInt, masterId);
        var dpthIndx = tauPoint(numDeps, tauRos, cosFctr);
        //numPrint(dpthIndx, 100, 100+i*20, zeroInt, zeroInt, zeroInt, masterId);
        var tempDisk = temp[0][dpthIndx];
        //numPrint(tempDisk, 100, 100+i*20, zeroInt, zeroInt, zeroInt, masterId);
        var tempDisk2 = Math.max(tempDisk, minTeff);
        //numPrint(tempDisk2, 150, 100 + i * 20, zeroInt, zeroInt, zeroInt, masterId);
        //       var logLamPeak = Math.log(wien / tempDisk2); // lambda in cm

        //var i =10;

        // var radiusPxI = Math.ceil((ii / 10.0) * radiusPx); // test 
        var radiusPxI = Math.ceil(radiusPx * Math.sin(Math.acos(cosFctr)));
        //console.log("i, radiusPxI, cosFctr, cosTheta[1][i]: "
        //        + i + " " + radiusPxI + " " + cosFctr + " " + cosTheta[1][i]);
        // var radiusPxI = Math.ceil(radiusPx * Math.sin(Math.acos(iCosThetaI)));
        var radiusStr = numToPxStrng(radiusPxI);

        if (i == 3) {
            var saveRadius = radiusPxI; // For HRD, plot nine
        }
        //numPrint(radiusPxI, 100, 100 + i * 20, zeroInt, zeroInt, zeroInt, masterId);

        // Adjust position to center star:
        // Radius is really the *diameter* of the symbol
        var xLowerYOffsetI = xLowerYOffset - radiusPxI / 2;
        var xLowerYOffsetStr = numToPxStrng(xLowerYOffsetI);
        var xOffsetI = xOffset - radiusPxI / 2;
        var xOffsetStr = numToPxStrng(xOffsetI);
        // limb darkening intensity factor:
        var dark = (intens[iLamMax][i] / intens[iLamMax][0]);
        //var dark = iIntMaxI;

        //var starRGBHex = colorTemp(tempDisk2, minTeff, maxTeff, dark);
        var RGBArr = colorTemp(tempDisk2, minTeff, maxTeff, dark);
        var rrI = RGBArr[0];
        var ggI = RGBArr[1];
        var bbI = RGBArr[2];
        if (i == 3) {
            var saveRGB = RGBArr; // For HRD, plot nine
        }
        //console.log("Plot seven: rrI, ggI, bbI: " + rrI + " " + ggI + " " + bbI);
        var starRGBHex = "rgb(" + rrI + "," + ggI + "," + bbI + ")";

        var starId = document.createElement("div");
        starId.class = "star";
        starId.style.display = "block";
        starId.style.position = "absolute";
        starId.style.height = radiusStr;
        starId.style.width = radiusStr;
        //   starId.style.border = tickBorder;
        starId.style.borderRadius = "100%";
        //starId.style.zIndex = "-1"; // put on top    
        starId.style.opacity = "1.0";
        starId.style.backgroundColor = starRGBHex;
        starId.style.marginTop = xLowerYOffsetStr;
        starId.style.marginLeft = xOffsetStr;
        //starId.style.border = "1px blue solid";

        plotSevenId.appendChild(starId);

        thet1 = 180.0 * Math.acos(cosTheta[1][i]) / Math.PI;
        thet2 = thet1.toPrecision(2);
        thet3 = thet2.toString(10);
        txtPrint("<span style='font-size:small; background-color:#888888'>" + thet3 + "</span>",
                titleXPos + (i + 2) * 20, titleYPos + 5, rrI, ggI, bbI, plotSevenId);
    }





    // Plot eight - Grotrian diagram for ionization stage and excitation level selected


    var plotRow = 2;
    var plotCol = 2;



    var numXTicks = 6;
    // Determine which ionization stage gas the majority population and scale the axis 
    /// with that population
    // From function levelPops():
    // logNums is a 2D 3 x numDeps array of logarithmic number densities
    // Row 0: neutral stage ground state population
    // Row 1: ionized stage ground state population
    // Row 2: level population of lower level of bb transition (could be in either stage I or II!) 
    if (logNums[0][tTau1] >= logNums[1][tTau1]) {
        var maxXData = logE * logNums[0][tTau1];
        var minXData = logE * logNums[1][tTau1];
    } else {
        var maxXData = logE * logNums[1][tTau1];
        var minXData = logE * logNums[0][tTau1];
    }

    var xAxisName = "<span title='Logarithmic number density of particles in lower E-level of b-b transition at &#964_Ros=1'>Log<sub>10</sub> <em>N</em><sub>l</sub>(&#964<sub>Ros</sub>=1) cm<sup>-3</sup></span>";


    var c = 2.9979249E+10; // light speed in vaccuum in cm/s
    var h = 6.62606957E-27; //Planck's constant in ergs sec
    var logC = Math.log(c);
    var logH = Math.log(h);
    var eV = 1.602176565E-12; // eV in ergs
    var logEv = Math.log(eV);
    //Log of line-center wavelength in cm
    var logLam0 = Math.log(lam0 * 1.0e-7);
    // energy of b-b transition

    var logTransE = logH + logC - logLam0 - logEv; // last term converts back to cgs units

    // Energy of upper E-level of b-b transition
    var chiU = chiL + Math.exp(logTransE);

    var numYTicks = 5;
    var minYData = 0.0;
    if (ionized) {
        var maxYData = chiI + chiU + 1.0; //eV
    } else {
        var maxYData = chiI + 1.0;
    }

    var yAxisName = "<span title='Atomic excitation energy'><a href='http://en.wikipedia.org/wiki/Excited_state' target='_blank'>Excitation<br /> E</a> (<a href='http://en.wikipedia.org/wiki/Electronvolt' target='_blank'>eV</a>)</span>";

    //(xRange, xOffset, yRange, yOffset, wColor, plotEightId);

    var xAxisParams = XAxis(plotRow, plotCol,
            numXTicks, minXData, maxXData, xAxisName,
            plotEightId);

    var yAxisParams = YAxis(plotRow, plotCol,
            numYTicks, minYData, maxYData, yAxisName,
            plotEightId);

    xOffset = xAxisParams[0];
    yOffset = xAxisParams[4];
    var rangeXData = xAxisParams[1];
    var deltaXData = xAxisParams[2];
    var deltaXPxl = xAxisParams[3];
    var rangeYData = yAxisParams[1];
    var deltaYData = yAxisParams[2];
    var deltaYPxl = yAxisParams[3];
    var xLowerYOffset = xAxisParams[5];


    // Second special "y-ticks" for lower and upper E-levels of b-b transition, and ground-state
    // ionization energy

    var yTickXOffset = xOffset - tickHeight / 2; //height and width reversed for y-ticks
    var yTickXOffsetStr = numToPxStrng(yTickXOffset);
    var yLabelXOffset = xOffset - 3 * labelHeight; //height & width reversed for y-ticks
    var yLabelXOffsetStr = numToPxStrng(yLabelXOffset);

    // From function levelPops():
    // logNums is a 2D 3 x numDeps array of logarithmic number densities
    // Row 0: neutral stage ground state population
    // Row 1: ionized stage ground state population
    // Row 2: level population of lower level of bb transition (could be in either stage I or II!) 
    // Row 3: level population of upper level of bb transition (could be in either stage I or II!) 
    var yData = [0.0, chiI, chiL, chiU];
    var yRightTickValStr = ["&#967<sub>0</sub>", "&#967<sub>I</sub>", "<span style='color:red'>&#967<sub>l</sub></span>", "&#967<sub>u</sub>"];
    // Offset for labelling on right of plot
    var yRightLabelXOffset = yLabelXOffset + xRange;
    var yRightLabelXOffset0 = yLabelXOffset + xRange + 25;
    var yRightLabelXOffsetStr = numToPxStrng(yRightLabelXOffset);
    var yRightLabelXOffset0Str = numToPxStrng(yRightLabelXOffset0);


    // No!:
    // Pointers into logNums rows must be in order of increasing atomic E:
    //   var lPoint = []; // declaration
    //   if (ionized) {
    //      lPoint = [0, 1, 2, 3];
    //   } else {
    //       lPoint = [0, 2, 3, 1];
    //   }

    //console.log("tTau1 = " + tTau1);
    for (var i = 0; i < yData.length; i++) {

        ii = 1.0 * i;
        //var yTickVal = minYData + (ii * deltaData);
        // var yTickRound = yTickVal.toPrecision(3);
        //var yTickValStr = yTickRound.toString(10);  // Not used, for now - ??


        var yTickPos = Math.floor(yRange * (yData[i] / rangeYData));
        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
        //stringify and add unit:
        var yShiftStr = numToPxStrng(yShift);
        // Make the y-tick mark, Teff:


        var yTickId = document.createElement("div");
        yTickId.class = "ytick";
        yTickId.style.position = "absolute";
        yTickId.style.display = "block";
        yTickId.style.marginLeft = yTickXOffsetStr;
        yTickId.style.border = tickBorder;
        // Note that for y ticks, the height and width are reversed!:
        yTickId.style.width = xRangeStr;
        yTickId.style.height = tickWidth;
        yTickId.style.zIndex = 0;
        yTickId.style.marginTop = yShiftStr;
        //Append the tickmark to the axis element
        plotEightId.appendChild(yTickId);
        // Now over-plot with the width of the "y-tickmark" scaled by the 
        // log number density in each E-level:
        var RGBHex = colHex(255, 0, 0);
        //var xRangePops = Math.floor(xRange * (logE*logNums[lPoint[i]][tTau1] / maxXData));
        var xRangePops = Math.floor(xRange * (logE * logNums[i][tTau1] / maxXData));
        var xRangePopsStr = numToPxStrng(xRangePops);
        var tickWidthPops = "3px";

        var yTickId = document.createElement("div");
        yTickId.class = "ytick";
        yTickId.style.position = "absolute";
        yTickId.style.display = "block";
        yTickId.style.marginLeft = yTickXOffsetStr;
        yTickId.style.border = tickBorder;
        // Note that for y ticks, the height and width are reversed!:
        yTickId.style.width = xRangePopsStr;
        yTickId.style.height = tickWidthPops;
        yTickId.style.zIndex = 0;
        yTickId.style.backgroundColor = RGBHex;
        yTickId.style.marginTop = yShiftStr;
        //Append the tickmark to the axis element
        plotEightId.appendChild(yTickId);
        //Make the y-tick label:

        var yLabelId = document.createElement("div");
        yLabelId.class = "ylabel";
        yLabelId.style.position = "absolute";
        yLabelId.style.display = "block";
        if (i == 0) {
            yLabelId.style.marginLeft = yRightLabelXOffset0Str;
        } else {
            yLabelId.style.marginLeft = yRightLabelXOffsetStr;
        }
        //    yLabelId.style.border = tickBorder; 
        yLabelId.style.marginTop = yShiftStr;
        // height and width are reversed for y labels
        yLabelId.style.width = labelHeightStr;
        yLabelId.style.height = labelWidth;
        yLabelId.style.zIndex = 0;
        //Insert the value of the tick mark: 
        yLabelId.innerHTML = "<span style='font-size:small'>" + yRightTickValStr[i] + "</span>";
        //Append the tick label to the axis element
        plotEightId.appendChild(yLabelId);
    }  // end y-tickmark loop, i

    // Add ionization stage labels:

    txtPrint("<span title='Singly ionized stage'>II</span>", xOffset + xRange - 15, xLowerYOffset - yRange, zeroInt, zeroInt, zeroInt, plotEightId);
    txtPrint("<span title='Neutral stage'>I</span>", xOffset + xRange - 15, xLowerYOffset - yRange / 2, zeroInt, zeroInt, zeroInt, plotEightId);

    // Transition:

    var opac = 1.0;
    var opacStr = numToPxStrng(opac);
    var r255 = 0;
    var g255 = 100;
    var b255 = 255; // gray
    var RGBHex = colHex(r255, g255, b255);
    // Vertical bar:
    var vBarHeight = yRange * (chiU - chiL) / rangeYData;
    var xTickPos = xRange / 3; // pixels
    var yTickPos = Math.floor(yRange * ((rangeYData - chiL) / rangeYData)) - vBarHeight;
    var xShift = xOffset + xTickPos;
    var xShiftStr = numToPxStrng(xShift);
    var yShift = (xLowerYOffset - yRange) + yTickPos;
    var yShiftStr = numToPxStrng(yShift);
    var vBarWidth = 3; //pixels 

    var vBarWidthStr = numToPxStrng(vBarWidth);
    var vBarHeightStr = numToPxStrng(vBarHeight);
    var vCrossId = document.createElement("div");
    vCrossId.class = "cross";
    vCrossId.style.position = "absolute";
    vCrossId.style.display = "block";
    vCrossId.style.height = vBarHeightStr;
    vCrossId.style.width = vBarWidthStr;
    //hCrossId.style.borderRadius = "100%";
    vCrossId.style.opacity = opacStr;
    vCrossId.style.backgroundColor = RGBHex;
    vCrossId.style.marginLeft = xShiftStr;
    vCrossId.style.marginTop = yShiftStr;
    //Append the dot to the plot
    plotEightId.appendChild(vCrossId);
    // Add title annotation:


    var titleYPos = xLowerYOffset - 1.15 * yRange;
    var titleXPos = 1.02 * xOffset;

    txtPrint("<span style='font-size:normal; color:blue'><a href='http://en.wikipedia.org/wiki/Grotrian_diagram' target='_blank'>Grotrian diagram</a></span>",
            titleXPos, titleYPos, zeroInt, zeroInt, zeroInt, plotEightId);



    // Plot Nine: HRDiagram


    var plotRow = 0;
    var plotCol = 0;

    var numXTicks = 6;
    // WARNING: Teff axis is backwards!!
    var minXData = logTen(30000); //K
    var maxXData = logTen(3000); //K


    var xAxisName = "<span title='Logarithmic surface temperature of spherical blackbody radiation emitter of equivalent bolometric surface flux, in Kelvins (K)'> \n\
<a href='http://en.wikipedia.org/wiki/Effective_temperature' target='_blank'>\n\
Log<sub>10 </sub><em>T</em><sub>eff</sub></a> \n\
(<a href='http://en.wikipedia.org/wiki/Kelvin' target='_blank'>K</a>)</span>";

    var numYTicks = 6;
    var minYData = -2.0; //solar luminosities;
    var maxYData = 3.5; //solar luminosities


    var yAxisName = "<span title='Logarithmic Bolometric luminosity'>\n\
<a href='http://en.wikipedia.org/wiki/Luminosity' target='_blank'>\n\
Log<sub>10</sub><em>L</em><sub>Bol</sub></a></span><br />  \n\
<span title='Solar luminosities'>\n\
<a href='http://en.wikipedia.org/wiki/Solar_luminosity' target='_blank'>\n\
<em>L</em><sub>Sun</sub></a></span> ";



    var xAxisParams = XAxis(plotRow, plotCol,
            numXTicks, minXData, maxXData, xAxisName,
            plotNineId);

    var yAxisParams = YAxis(plotRow, plotCol,
            numYTicks, minYData, maxYData, yAxisName,
            plotNineId);

    xOffset = xAxisParams[0];
    yOffset = yAxisParams[4];
    var rangeXData = xAxisParams[1];
    var deltaXData = xAxisParams[2];
    var deltaXPxl = xAxisParams[3];
    var rangeYData = yAxisParams[1];
    var deltaYData = yAxisParams[2];
    var deltaYPxl = yAxisParams[3];
    var xLowerYOffset = xAxisParams[5];


    var titleYPos = xLowerYOffset - 1.15 * yRange;
    var titleXPos = 1.02 * xOffset;

    txtPrint("<span style='font-size:normal; color:blue'><a href='http://www.ap.smu.ca/~ishort/hrdtest3.html' target='_blank'>H-R Diagram</a></span>",
            titleXPos, titleYPos, zeroInt, zeroInt, zeroInt, plotNineId);

    // *********  Input stellar data

//Sun

    var sunClass = "G2";
    var sunTeff = 5778;
// var sunTeff = 10000; //debug test
    var sunB_V = 0.656;
    var sunM_v = 4.83;
    var sunM_Bol = 4.75;
    var sunRad = 1.0;
// var sunRad = 10.0; //debug test
    var logSunLum = 2.5 * logTen(1.0); //log Suns luminosity in solar luminosities 


// Carroll & Ostlie, 2nd Ed. , Appendix G:

//Main sequence

    var msClass = ["O5", "O6", "O7", "O8", "B0", "B1", "B2", "B3", "B5", "B6", "B7", "B8", "B9", "A0", "A1", "A2", "A5", "A8", "F0", "F2", "F5", "F8", "G0", "G2", "G8", "K0", "K1", "K3", "K4", "K5", "K7", "M0", "M1", "M2", "M3", "M4", "M5", "M6", "M7"];
    var msTeffs = [42000, 39500, 37500, 35800, 30000, 25400, 20900, 18800, 15200, 13700, 12500, 11400, 10500, 9800, 9400, 9020, 8190, 7600, 7300, 7050, 6650, 6250, 5940, 5790, 5310, 5150, 4990, 4690, 4540, 4410, 4150, 3840, 3660, 3520, 3400, 3290, 3170, 3030, 2860];
    var msB_V = [-0.33, -0.33, -0.32, -0.32, -0.30, -0.26, -0.24, -0.20, -0.17, -0.15, -0.13, -0.11, -0.07, -0.02, +0.01, +0.05, +0.15, +0.25, +0.30, +0.35, +0.44, +0.52, +0.58, +0.63, +0.74, +0.81, +0.86, +0.96, +1.05, +1.15, +1.33, +1.40, +1.46, +1.49, +1.51, +1.54, +1.64, +1.73, +1.80];
    var msM_v = [-5.1, -5.1, -4.9, -4.6, -3.4, -2.6, -1.6, -1.3, -0.5, -0.1, +0.3, +0.6, +0.8, +1.1, +1.3, +1.5, +2.2, +2.7, +3.0, +3.4, +3.9, +4.3, +4.7, +4.9, +5.6, +5.7, +6.0, +6.5, +6.7, +7.1, +7.8, +8.9, +9.6, 10.4, 11.1, 11.9, 12.8, 13.8, 14.7];
    var msBC = [-4.40, -3.93, -3.68, -3.54, -3.16, -2.70, -2.35, -1.94, -1.46, -1.21, -1.02, -0.80, -0.51, -0.30, -0.23, -0.20, -0.15, -0.10, -0.09, -0.11, -0.14, -0.16, -0.18, -0.20, -0.40, -0.31, -0.37, -0.50, -0.55, -0.72, -1.01, -1.38, -1.62, -1.89, -2.15, -2.38, -2.73, -3.21, -3.46];
    var msMass = [60, 37, 29, 23, 17.5, 13.0, 10.0, 7.6, 5.9, 4.8, 4.3, 3.8, 3.2, 2.9, 2.5, 2.2, 2.0, 1.7, 1.6, 1.5, 1.4, 1.2, 1.05, 0.90, 0.83, 0.79, 0.75, 0.72, 0.69, 0.67, 0.55, 0.51, 0.43, 0.40, 0.33, 0.26, 0.21, 0.13, 0.10];

//var msRads = [13.4 ,12.2 ,11.0 ,10.0 , 6.7 , 5.2 , 4.1 , 3.8 , 3.2 , 2.9 , 2.7 , 2.5 , 2.3 , 2.2 , 2.1 , 2.0 , 1.8 , 1.5 , 1.4 , 1.3 , 1.2 , 1.1 , 1.06, 1.03, 0.96, 0.93, 0.91, 0.86, 0.83, 0.80, 0.74, 0.63, 0.56, 0.48, 0.41, 0.35, 0.29, 0.24, 0.20];
//var msM_Bol = [-9.51,-9.04,-8.60,-8.18,-6.54,-5.26,-3.92,-3.26,-1.96,-1.35,-0.77,-0.22,+0.28,+0.75,+1.04,+1.31,+2.02,+2.61,+2.95,+3.27,+3.72,+4.18,+4.50,+4.66,+5.20,+5.39,+5.58,+5.98,+6.19,+6.40,+6.84,+7.52,+7.99,+8.47,+8.97,+9.49,10.1 ,10.6 ,11.3];

//Needed for tick mark placement in pixels:

    var maxMsTeff = msTeffs[0];
    var minMsTeff = msTeffs[msTeffs.length - 1];

    var maxMsM_v = msM_v[msM_v.length - 1];
    var minMsM_v = msM_v[0];

// Main sequence data processing:


    var msNum = msClass.length;

    var msM_Bol = new Array();
    var logR45 = new Array();
    var logR = new Array();
    var msRads = new Array();
    var msLogLum = new Array();


    msM_Bol.length = msNum;
    logR45.length = msNum;
    logR.length = msNum;
    msRads.length = msNum;
    msLogLum.length = msNum;


// Calculate radii in solar radii:
// For MS stars, do the Luminosity as well

    for (var i = 0; i < msNum; i++) {

        msM_Bol[i] = msM_v[i] + msBC[i];
        var msTeffSol = msTeffs[i] / sunTeff;

        logR45[i] = 2.5 * logSunLum + sunM_Bol - 10.0 * logTen(msTeffSol) - msM_Bol[i];
        logR[i] = logR45[i] / 4.5;
        msRads[i] = Math.exp(Math.LN10 * logR[i]);  //No base ten exponentiation in JS!
        var msLogL = (sunM_Bol - msM_Bol[i]) / 2.5;
// Round log(Lum) to 1 decimal place:
        msLogL = 10.0 * msLogL;
        msLogL = Math.floor(msLogL);
        msLogLum[i] = msLogL / 10.0;


    } // end i loop


// Giants:

    var rgbClass = ["O5", "O6", "O7", "O8", "B0", "B1", "B2", "B3", "B5", "B6", "B7", "B8", "B9", "A0", "A1", "A2", "A5", "A8", "F0", "F2", "F5", "G0", "G2", "G8", "K0", "K1", "K3", "K4", "K5", "K7", "M0", "M1", "M2", "M3", "M4", "M5", "M6"];
    var rgbTeffs = [39400, 37800, 36500, 35000, 29200, 24500, 20200, 18300, 15100, 13800, 12700, 11700, 10900, 10200, 9820, 9460, 8550, 7830, 7400, 7000, 6410, 5470, 5300, 4800, 4660, 4510, 4260, 4150, 4050, 3870, 3690, 3600, 3540, 3480, 3440, 3380, 3330];
    var rgbB_V = [-0.32, -0.32, -0.32, -0.31, -0.29, -0.26, -0.24, -0.20, -0.17, -0.15, -0.13, -0.11, -0.07, -0.03, +0.01, +0.05, +0.15, +0.25, +0.30, +0.35, +0.43, +0.65, +0.77, +0.94, +1.00, +1.07, +1.27, +1.38, +1.50, +1.53, +1.56, +1.58, +1.60, +1.61, +1.62, +1.63, +1.52];
    var rgbM_v = [-5.9, -5.7, -5.6, -5.5, -4.7, -4.1, -3.4, -3.2, -2.3, -1.8, -1.4, -1.0, -0.6, -0.4, -0.2, -0.1, +0.6, +1.0, +1.3, +1.4, +1.5, +1.3, +1.3, +1.0, +1.0, +0.9, +0.8, +0.8, +0.7, +0.4, +0.0, -0.2, -0.4, -0.4, -0.4, -0.4, -0.4];
    var rgbBC = [-4.05, -3.80, -3.58, -3.39, -2.88, -2.43, -2.02, -1.60, -1.30, -1.13, -0.97, -0.82, -0.71, -0.42, -0.29, -0.20, -0.14, -0.10, -0.11, -0.11, -0.14, -0.20, -0.27, -0.42, -0.50, -0.55, -0.76, -0.94, -1.02, -1.17, -1.25, -1.44, -1.62, -1.87, -2.22, -2.48, -2.73];

//var rgbRads = [18.5,16.8,15.4,14.3,11.4,10.0, 8.6, 8.0, 6.7, 6.1, 5.5, 5.0, 4.5, 4.1, 3.9, 3.7, 3.3, 3.1, 3.2, 3.3, 3.8, 6.0, 6.7, 9.6,10.9,12.5,16.4,18.7,21.4,27.6,39.3,48.6,58.5,69.7,82.0,96.7,16];
//var rgbM_Bol = [-9.94,-9.55,-9.20,-8.87,-7.58,-6.53,-5.38,-4.78,-3.56,-2.96,-2.38,-1.83,-1.31,-0.83,-0.53,-0.26,+0.44,+0.95,+1.17,+1.31,+1.37,+1.10,+1.00,+0.63,+0.48,+0.32,-0.01,-0.18,-0.36,-0.73,-1.28,-1.64,-1.97,-2.28,-2.57,-2.86,-3.18];

// RGB sequence data processing:


    var rgbNum = rgbClass.length;

    var rgbM_Bol = new Array();
    var logR45 = new Array();
    var logR = new Array();
    var rgbRads = new Array();
    var rgbLogLum = new Array();

    rgbM_Bol.length = rgbNum;
    logR45.length = rgbNum;
    logR.length = rgbNum;
    rgbRads.length = rgbNum;


// Calculate radii in solar radii:

    for (var i = 0; i < rgbNum; i++) {

        rgbM_Bol[i] = rgbM_v[i] + rgbBC[i];
        var rgbTeffSol = rgbTeffs[i] / sunTeff;

        logR45[i] = 2.5 * logSunLum + sunM_Bol - 10.0 * logTen(rgbTeffSol) - rgbM_Bol[i];
        logR[i] = logR45[i] / 4.5;
        rgbRads[i] = Math.exp(Math.LN10 * logR[i]);  //No base ten exponentiation in JS!

        var rgbLogL = (sunM_Bol - rgbM_Bol[i]) / 2.5;
// Round log(Lum) to 1 decimal place:
        rgbLogL = 10.0 * rgbLogL;
        rgbLogL = Math.floor(rgbLogL);
        rgbLogLum[i] = rgbLogL / 10.0;

    } // end i loop


    /* No! Too bright for what GrayStar can model!
     //Supergiants:
     
     var sgbClass = ["O5", "O6", "O7", "O8", "B0", "B1", "B2", "B3", "B5", "B6", "B7", "B8", "B9", "A0", "A1", "A2", "A5", "A8", "F0", "F2", "F5", "F8", "G0", "G2", "G8", "K0", "K1", "K3", "K4", "K5", "K7", "M0", "M1", "M2", "M3", "M4", "M5", "M6"];
     var sgbTeffs = [40900, 38500, 36200, 34000, 26200, 21400, 17600, 16000, 13600, 12600, 11800, 11100, 10500, 9980, 9660, 9380, 8610, 7910, 7460, 7030, 6370, 5750, 5370, 5190, 4700, 4550, 4430, 4190, 4090, 3990, 3830, 3620, 3490, 3370, 3210, 3060, 2880, 2710];
     var sgbB_V = [-0.31, -0.31, -0.31, -0.29, -0.23, -0.19, -0.17, -0.13, -0.10, -0.08, -0.05, -0.03, -0.02, -0.01, +0.02, +0.03, +0.09, +0.14, +0.17, +0.23, +0.32, +0.56, +0.76, +0.87, +1.15, +1.24, +1.30, +1.46, +1.53, +1.60, +1.63, +1.67, +1.69, +1.71, +1.69, +1.76, +1.80, +1.86];
     var sgbM_v = [-6.5, -6.5, -6.6, -6.6, -6.9, -6.9, -6.7, -6.7, -6.6, -6.4, -6.3, -6.3, -6.3, -6.3, -6.3, -6.3, -6.3, -6.4, -6.4, -6.4, -6.4, -6.4, -6.3, -6.3, -6.1, -6.1, -6.0, -5.9, -5.8, -5.7, -5.6, -5.8, -5.8, -5.8, -5.5, -5.2, -4.8, -4.9];
     var sgbBC = [-3.87, -3.74, -3.48, -3.35, -2.49, -1.87, -1.58, -1.26, -0.95, -0.88, -0.78, -0.66, -0.52, -0.41, -0.32, -0.28, -0.13, -0.03, -0.01, 0.00, -0.03, -0.09, -0.15, -0.21, -0.42, -0.50, -0.56, -0.75, -0.90, -1.01, -1.20, -1.29, -1.38, -1.62, -2.13, -2.75, -3.47, -3.90];
     
     //var sgbRads = [21,  22,  23,  25,  31,  37,  42,  45,  51,  53,  56,  58,  61,  64,  67,  69,  78,  91, 102, 114, 140, 174, 202, 218, 272, 293, 314, 362, 386, 415, 473, 579, 672, 791, 967,1220,1640,2340];
     //var sgbM_Bol = [-10.4,-10.2,-10.1, -9.9, -9.3, -8.8, -8.2, -7.9, -7.5, -7.3, -7.1, -6.9, -6.8, -6.7, -6.6, -6.5, -6.4, -6.4, -6.4, -6.4, -6.4, -6.4, -6.4, -6.4, -6.5, -6.5, -6.5, -6.6, -6.7, -6.7, -6.8, -7.0, -7.2, -7.4, -7.6, -7.9, -8.3, -8.8];
     
     // SGB sequence data processing:
     
     
     var sgbNum = sgbClass.length;
     
     var sgbM_Bol = new Array();
     var logR45 = new Array();
     var logR = new Array();
     var sgbRads = new Array();
     var sgbLogLum = new Array();
     
     sgbM_Bol.length = sgbNum;
     logR45.length = sgbNum;
     logR.length = sgbNum;
     sgbRads.length = sgbNum;
     
     
     // Calculate radii in solar radii:
     
     for (var i = 0; i < sgbNum; i++) {
     
     sgbM_Bol[i] = sgbM_v[i] + sgbBC[i];
     var sgbTeffSol = sgbTeffs[i] / sunTeff;
     
     logR45[i] = 2.5 * logSunLum + sunM_Bol - 10.0 * logTen(sgbTeffSol) - sgbM_Bol[i];
     logR[i] = logR45[i] / 4.5;
     sgbRads[i] = Math.exp(Math.LN10 * logR[i]);  //No base ten exponentiation in JS!
     
     var sgbLogL = (sunM_Bol - sgbM_Bol[i]) / 2.5;
     // Round log(Lum) to 1 decimal place:
     sgbLogL = 10.0 * sgbLogL;
     sgbLogL = Math.floor(sgbLogL);
     sgbLogLum[i] = sgbLogL / 10.0;
     
     } // end i loop
     */

    //Data loops - plot the result!

    //MS stars

    var dSize = 2.0; //plot point size
    var opac = 0.7; //opacity
    // RGB color
    var r255 = 50;
    var g255 = 50;
    var b255 = 50; //dark gray

    var ii;
    for (var i = 5; i < msNum - 3; i++) {

        ii = 1.0 * i;
        var xTickPos = xRange * (logTen(msTeffs[i]) - minXData) / rangeXData; // pixels   

        // horizontal position in pixels - data values increase rightward:
        var xShift = xOffset + xTickPos;
        ////stringify and add unit:
        //        var xShiftStr = numToPxStrng(xShift);

        var yTickPos = yRange * (msLogLum[i] - minYData) / rangeYData;
        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
        ////stringify and add unit:
        //       var yShiftStr = numToPxStrng(yShift);

        plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotNineId);


    }


    //RGB stars

    // RGB color
    var r255 = 100;
    var g255 = 100;
    var b255 = 100; //gray

    var ii;
    for (var i = 4; i < rgbNum - 2; i++) {

        ii = 1.0 * i;
        var xTickPos = xRange * (logTen(rgbTeffs[i]) - minXData) / rangeXData; // pixels   

        // horizontal position in pixels - data values increase rightward:
        var xShift = xOffset + xTickPos;
        ////stringify and add unit:
        //        var xShiftStr = numToPxStrng(xShift);

        var yTickPos = yRange * (rgbLogLum[i] - minYData) / rangeYData;
        // vertical position in pixels - data values increase upward:
        var yShift = xLowerYOffset - yTickPos;
        ////stringify and add unit:
        //       var yShiftStr = numToPxStrng(yShift);

        plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotNineId);


    }

    /* No! Too bright for what GrayStar can model!
     //SGB stars
     
     // RGB color
     var r255 = 150;
     var g255 = 150;
     var b255 = 150; //light gray
     
     var ii;
     for (var i = 4; i < sgbNum - 3; i++) {
     
     ii = 1.0 * i;
     var xTickPos = xRange * (logTen(sgbTeffs[i]) - minXData) / rangeXData; // pixels   
     
     // horizontal position in pixels - data values increase rightward:
     var xShift = xOffset + xTickPos;
     ////stringify and add unit:
     //        var xShiftStr = numToPxStrng(xShift);
     
     var yTickPos = yRange * (sgbLogLum[i] - minYData) / rangeYData;
     // vertical position in pixels - data values increase upward:
     var yShift = xLowerYOffset - yTickPos;
     ////stringify and add unit:
     //       var yShiftStr = numToPxStrng(yShift);
     
     plotPnt(xShift, yShift, r255, g255, b255, opac, dSize, plotNineId);
     
     
     }
     */


    // Now overplot our star:
    //console.log("teff, bolLum, radiusPxI, rrI, ggI, bbI: " +
    //        teff + " " + bolLum + " " + radiusPxI + " " + rrI + " " + ggI + " " + bbI);
    var xTickPos = xRange * (logTen(teff) - minXData) / rangeXData; // pixels   
    // horizontal position in pixels - data values increase rightward:
    var xShift = xOffset + xTickPos;
    var yTickPos = yRange * (logTen(bolLum) - minYData) / rangeYData;
    // vertical position in pixels - data values increase upward:
    var yShift = xLowerYOffset - yTickPos;
    //Take color and radius from the last step of the star rendering loop (plot Seve) - that should be the inner-most disk:
    var radiusPxThis = saveRadius / 5;
    var rrI = saveRGB[0];
    var ggI = saveRGB[1];
    var bbI = saveRGB[2];
    plotPnt(xShift, yShift, zeroInt, zeroInt, zeroInt, opac, 1.05 * radiusPxThis, plotNineId); //black border to bring attention
    plotPnt(xShift, yShift, rrI, ggI, bbI, opac, radiusPxThis, plotNineId);


    //Now overplot Luminosity class markers:

//    //I
//    xShift = xOffset + xRange * (logTen(sgbTeffs[sgbNum - 2]) - minXData) / rangeXData; // pixels 
//    yShift = xLowerYOffset - (yRange * (sgbLogLum[sgbNum - 1] - minYData) / rangeYData);
//    txtPrint("<span style='font-size:normal'><a href='http://en.wikipedia.org/wiki/Stellar_classification' target='_blank'>\n\
// I</a></span>", xShift, yShift, 50, 50, 50, plotNineId);
    //III
    xShift = xOffset + xRange * (logTen(rgbTeffs[rgbNum - 8]) - minXData) / rangeXData; // pixels 
    yShift = xLowerYOffset - (yRange * (rgbLogLum[rgbNum - 8] - minYData) / rangeYData);
    txtPrint("<span style='font-size:normal'><a href='http://en.wikipedia.org/wiki/Stellar_classification' title='Giants' target='_blank'>\n\
III</a></span>", xShift, yShift, 50, 50, 50, plotNineId);
    //V
    xShift = xOffset + xRange * (logTen(msTeffs[msNum - 8]) - minXData) / rangeXData; // pixels 
    yShift = xLowerYOffset - (yRange * (msLogLum[msNum - 8] - minYData) / rangeYData);
    txtPrint("<span style='font-size:normal'><a href='http://en.wikipedia.org/wiki/Stellar_classification' title='Main Sequence, Dwarfs' target='_blank'>\n\
V</a></span>", xShift, yShift, 50, 50, 50, plotNineId);


// * * * * * *
// Debug ouput:

    /*
     numPrint(teff, 100, 100, zeroInt, zeroInt, zeroInt, textId); //debug
     numPrint(logg, 100, 120, zeroInt, zeroInt, zeroInt, textId); //debug
     numPrint(kappaScale, 100, 140, zeroInt, zeroInt, zeroInt, textId); //debug
     
     numPrint(lam0, 100, 180, zeroInt, zeroInt, zeroInt, textId); //debug
     numPrint(xiT, 100, 200, zeroInt, zeroInt, zeroInt, textId); //debug
     numPrint(mass, 100, 220, zeroInt, zeroInt, zeroInt, textId); //debug
     numPrint(logGammaCol, 100, 240, zeroInt, zeroInt, zeroInt, textId); //debug
     numPrint(logN, 100, 260, zeroInt, zeroInt, zeroInt, textId); //debug
     numPrint(logF, 100, 280, zeroInt, zeroInt, zeroInt, textId); //debug
     numPrint(chiI, 100, 300, zeroInt, zeroInt, zeroInt, textId); //debug
     numPrint(chiL, 100, 320, zeroInt, zeroInt, zeroInt, textId); //debug
     */


    /*
     * 
     /*
     for (var i = 0; i < numDeps; i++) {
     
     remain = (logE * tauRos[1][i] * 1000.0) % 10;
     roundNum = (logE * tauRos[1][i]) - (remain / 1000.0);
     numPrint(roundNum, 100, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug 
     //numPrint(tauRos[0][i], 400, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug
     remain = (temp[0][i] * 1000.0) % 10;
     roundNum = temp[0][i] - (remain / 1000.0);
     numPrint(roundNum, 300, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug
     remain = (logE * kappa[1][i] * 1000.0) % 10;
     roundNum = (logE * kappa[1][i]) - (remain / 1000.0);
     numPrint(roundNum, 500, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug
     remain = (logE * press[1][i] * 1000.0) % 10;
     roundNum = (logE * press[1][i]) - (remain / 1000.0);
     numPrint(roundNum, 700, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug
     remain = (logE * rho[1][i] * 1000.0) % 10;
     roundNum = (logE * rho[1][i]) - (remain / 1000.0);
     numPrint(roundNum, 900, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug
     remain = ((depths[i]*1.0E-5) * 1000.0) % 10;
     roundNum = (depths[i]*1.0E-5) - (remain / 1000.0);
     numPrint(roundNum, 1100, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug
     
     }
     */

    /*
     for (var i = 0; i < numLams; i++) {
     remain = (lambdaScale[i] * 1000.0) % 10;
     roundNum = (lambdaScale[i]) - (remain / 1000.0);
     numPrint(roundNum, 100, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug 
     remain = (intensLam[0] * 1000.0) % 10;
     roundNum = (intensLam[0]) - (remain / 1000.0);
     numPrint(roundNum, 300, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug 
     //remain = (flux[1][i] * 1000.0) % 10;
     //roundNum = (flux[1][i]) - (remain / 1000.0);
     //numPrint(roundNum, 100, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug 
     }
     */

    /*
     for (var i = 0; i < numThetas; i++) {
     remain = (cosTheta[0][i] * 100000.0) % 10;
     roundNum = (cosTheta[0][i]) - (remain / 100000.0);
     numPrint(roundNum, 300, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug 
     remain = (cosTheta[1][i] * 100000.0) % 10;
     roundNum = (cosTheta[1][i]) - (remain / 100000.0);
     numPrint(roundNum, 500, 50 + 15 * i, zeroInt, zeroInt, zeroInt, textId); //debug 
     }
     */

    /*
     remain = (teffFlux * 1000.0) % 10;
     roundNum = (teffFlux) - (remain / 1000.0);
     numPrint(roundNum, 100, 100, zeroInt, zeroInt, zeroInt, textId); //debug
     
     for (var i = 0; i < colors.length; i++) {
     remain = (colors[i] * 1000.0) % 10;
     roundNum = (colors[i]) - (remain / 1000.0);
     numPrint(roundNum, 200, 100 + i * 15, zeroInt, zeroInt, zeroInt, textId);
     }
     */



    /*
     remain = (Wlambda * 1000.0) % 10;
     roundNum = (Wlambda) - (remain / 1000.0);
     numPrint(roundNum, 100, 300, masterId, zeroInt, zeroInt, zeroInt, textId); //debug
     */

    // txtPrint("Test", 200, 800, zeroInt, zeroInt, zeroInt, masterId);

}
; //end function main()
